from flask import Blueprint, render_template, request, session, send_from_directory, current_app
import os, tempfile, subprocess, json, glob
from PyQt5 import QtCore

main = Blueprint('main', __name__)

showStatusSignal = None
kpyPath = 'kpy'
local_ip = ''


def setStatusSignal(sig):
	global showStatusSignal
	showStatusSignal = sig


def setKpyPath(pth, ip):
	global kpyPath, local_ip
	kpyPath = pth
	local_ip = ip


def is_c(code):
	assembly_keywords = ['mov', 'add', 'sub', 'jmp', 'ret', 'LDI', 'STS', 'SUB', 'LDS', 'main:', 'loop:', '.section',
	                     '.text']
	c_keywords = ['int', 'char', 'float', 'double', 'void', 'if', 'else', 'for', 'while', 'return']

	assembly_count = sum(keyword in code for keyword in assembly_keywords)
	c_count = sum(keyword in code for keyword in c_keywords)

	if assembly_count > c_count:
		return False
	elif c_count > assembly_count:
		return True


@main.route('/')
def index():
	return render_template('index.html')


@main.route('/favicon.ico')
def favicon():
	return send_from_directory(os.path.join(current_app.root_path, 'static'), 'logo.png', mimetype='image/png')


@main.route('/compile', methods=['GET', 'POST'])
def mycompiler():
	global showStatusSignal, kpyPath
	try:
		print('compile...')
		extension = '.S'
		processor = 'atmega32'
		if 'processor' in request.form:
			processor = request.form.get('processor')
			print('processor specified', processor)

		if is_c(request.form.get('code')):
			extension = '.c'
		tfile = tempfile.NamedTemporaryFile(mode='w+', dir=kpyPath, suffix=extension)
		tfile.write(request.form.get('code'))
		tfile.flush()

		name = tfile.name.replace(extension, '')
		cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s%s"' % (processor, name, name, extension)
		res = subprocess.getstatusoutput(cmd)
		showStatusSignal.emit(f'Compiler Active at {local_ip},   Client: {str(request.remote_addr)}', False)
		if (res[0] == 1):  # Error
			return {'status': False, 'msg': 'compile error: ' + res[1], 'raw': request.form.get('code')}

		cmd = 'avr-objcopy -j .text -j .data -O ihex "%s" "%s.hex"' % (name, name)
		res2 = subprocess.getstatusoutput(cmd)
		if (res2[0] == 1):  # Error
			return {'status': False, 'hex': '', 'msg': 'objcopy error: ' + res2[1]}

		hexfile = open(name + '.hex', 'r')
		hx = hexfile.read()

		# Cleanup
		full_pattern = os.path.join(kpyPath, name + "*")
		matching_files = glob.glob(full_pattern)
		# Delete each matching file tmp file.
		for file_path in matching_files:
			if not file_path.endswith(extension):
				try:
					os.remove(file_path)
				# print(f"Deleted: {file_path}")
				except Exception as e:
					print(f"Error deleting {file_path}: {e}")

		return {'status': True, 'hex': json.dumps(hx)}
	except Exception as e:
		return {'status': False, 'msg': 'error: ' + str(e)}



from PyQt5 import QtGui,QtCore,QtWidgets
from .templates import ui_dio,ui_dio_pwm,ui_dio_adc,ui_dio_adcLog,ui_dio_sensor,ui_regvals,ui_dio_cntr,ui_regedit,ui_dio_control
from .templates import ui_dio_robot,ui_dio_stepper,ui_dummyio

from . import REGISTERS
from .templates.gauge import Gauge
import numpy as np
from functools import partial
from collections import OrderedDict
import time

colors=[(0,255,0),(255,0,0),(255,255,100),(10,255,255)]+[(50+np.random.randint(200),50+np.random.randint(200),150+np.random.randint(100)) for a in range(10)]

try:
	from scipy import optimize
except:
	print('scipy not available')


def find_peak(va):
	vmax = 0.0
	size = len(va)
	index = 0
	for i in range(1,size):		# skip first 2 channels, DC
		if va[i] > vmax:
			vmax = va[i]
			index = i
	return index

#-------------------------- Fourier Transform ------------------------------------
def fft(ya, si):
	'''
	Returns positive half of the Fourier transform of the signal ya. 
	Sampling interval 'si', in milliseconds
	'''
	NP = len(ya)
	if NP%2: #odd number
		ya = ya[:-1]
		NP-=1
	v = np.array(ya)
	tr = abs(np.fft.fft(v))/NP
	frq = np.fft.fftfreq(NP, si)
	x = frq.reshape(2,int(NP/2))
	y = tr.reshape(2,int(NP/2))
	return x[0], y[0]    

def find_frequency(x,y):		# Returns the fundamental frequency using FFT
	tx,ty = fft(y, x[1]-x[0])
	index = find_peak(ty)
	if index == 0:
		return None
	else:
		return tx[index]

#-------------------------- Sine Fit ------------------------------------------------
def sine_eval(x,p):			# y = a * sin(2*pi*f*x + phi)+ offset
	return p[0] * np.sin(2*np.pi*p[1]*x+p[2])-p[3]

def sine_erf(p,x,y):					
	return y - sine_eval(x,p)


def fit_sine(xa,ya, freq = 0):	# Time in mS, V in volts, freq in Hz, accepts numpy arrays
	size = len(ya)
	mx = max(ya)
	mn = min(ya)
	amp = (mx-mn)/2
	if freq == 0:						# Guess frequency not given
		freq = find_frequency(xa,ya)
	if freq == None:
		return None
	#print 'guess a & freq = ', amp, freq
	par = [amp, freq, 0.0, 0.0] # Amp, freq, phase , offset
	par, pcov = optimize.leastsq(sine_erf, par, args=(xa, ya))
	return par
	

#--------------------------Damped Sine Fit ------------------------------------------------
def dsine_eval(x,p):
	return     p[0] * np.sin(2*np.pi*p[1]*x+p[2]) * np.exp(-p[4]*x) - p[3]
def dsine_erf(p,x,y):
	return y - dsine_eval(x,p)


def fit_dsine(xlist, ylist, freq = 0):
	size = len(xlist)
	xa = np.array(xlist, dtype=np.float)
	ya = np.array(ylist, dtype=np.float)
	amp = (max(ya)-min(ya))/2
	off = np.average(ya)
	if freq == 0:
		freq = find_frequency(xa,ya)
	par = [amp, freq, 0.0, off, 0.1] # Amp, freq, phase , offset, decay constant
	par, pcov = optimize.leastsq(dsine_erf, par,args=(xa,ya))
	if pcov > 4:
		return None
	return par


def widget(name,Q,**kwargs):
	if 'OC' in kwargs.get('extra',''):
		widget = DIOPWM(name,Q,**kwargs)
	elif 'ADC' in kwargs.get('extra',''):
		widget = DIOADC(name,Q,**kwargs)
	elif kwargs.get('extra','') in ['T1']: #Counters
		widget = DIOCNTR(name,Q,**kwargs)
	elif kwargs.get('extra','') == 'FIXED': #Uneditable
		widget = DIODUMMY(name,**kwargs)
	else: #Regular D IO
		widget = DIO(name,Q,**kwargs)
	widget.setProperty("class", name)
	return widget

class REGVALS(QtWidgets.QFrame,ui_regvals.Ui_Frame):
	def __init__(self,name):
		super(REGVALS, self).__init__()
		self.setupUi(self)
		self.name = name
		self.DDR = 'DDR'+name
		self.PORT = 'PORT'+name
		self.PIN  = 'PIN' + name
		self.reglist = {}
		self.format = '{0:d}'
		self.L1.setText('%s\n-'%(self.DDR))
		self.L2.setText('%s\n-'%(self.PORT))
		self.L3.setText('%s\n-'%(self.PIN))

	def setRegs(self,reglist):
		if self.DDR in reglist:
			self.L1.setText('%s\n%s'%(self.DDR,self.format.format(reglist.get(self.DDR))) )
		if self.PORT in reglist:
			self.L2.setText('%s\n%s'%(self.PORT,self.format.format(reglist.get(self.PORT))) )
		if self.PIN in reglist:
			self.L3.setText('%s\n%s'%(self.PIN,self.format.format(reglist.get(self.PIN))) )
		self.reglist = reglist

	def mousePressEvent(self, event):
		if self.format == '{0:d}':
			self.format = '{0:08b}'
		elif self.format == '{0:08b}':
			self.format = '0x{0:02X}'
		elif self.format == '0x{0:02X}':
			self.format = '{0:d}'
		self.setRegs(self.reglist)

class DIO(QtWidgets.QStackedWidget,ui_dio.Ui_stack):
	def __init__(self,name,Q,**kwargs):
		super(DIO, self).__init__()
		self.setupUi(self)
		self.name = name
		self.Q = Q #Command Queue
		self.nameOut.setText(name)
		self.nameIn.setText(name)
		self.nameIn.setEnabled(False)
		self.currentPage = 0
		
	def next(self):
		self.currentPage+=1
		if self.currentPage >= self.count():
			self.currentPage = 0
		self.setCurrentIndex(self.currentPage)
		self.initPage()

	def initPage(self):
		if self.currentPage == 0: #Input
			self.Q.append(['DTYPE',self.name,0])
		elif self.currentPage == 1: #Output
			self.Q.append(['DTYPE',self.name,1])

	def setOutputState(self,state):
		self.nameOut.setChecked(state)
		self.pullup.setChecked(state)
		self.Q.append(['DSTATE',self.name,state])

class DIODUMMY(QtWidgets.QWidget,ui_dummyio.Ui_Form):
	def __init__(self,name,**kwargs):
		super(DIODUMMY, self).__init__()
		self.setupUi(self)
		self.name = name
		self.nameIn.setText(name)
		

class DIOPWM(QtWidgets.QStackedWidget,ui_dio_pwm.Ui_stack):
	def __init__(self,name,Q,**kwargs):
		super(DIOPWM, self).__init__()
		self.setupUi(self)
		self.name = name
		self.type = kwargs.get('extra','')
		if self.type == 'OC0':
			self.slider.setMaximum(255)
		elif self.type == 'OC2':
			self.slider.setMaximum(255)
		elif self.type == 'OC1A':
			self.slider.setMaximum(1023)
		
		self.Q = Q #Command Queue
		self.nameOut.setText(name)
		self.nameIn.setText(name)
		self.nameIn.setEnabled(False)
		self.currentPage = 0
		
	def next(self):
		if self.currentPage == 2: #Disable PWM
			if self.type == 'OC0':
				self.Q.append(['WRITE','TCCR0',0])
			elif self.type == 'OC2':
				self.Q.append(['WRITE','TCCR2',0])
				self.Q.append(['WRITE','TCNT2',0])
			elif self.type == 'OC1A':
				self.Q.append(['WRITE','TCCR1A',0])
				self.Q.append(['WRITE','TCCR1B',0])
		self.currentPage+=1
		if self.currentPage >= self.count():
			self.currentPage = 0
		self.setCurrentIndex(self.currentPage)
		self.initPage()

	def initPage(self):
		if self.currentPage == 2: #PWM
			self.Q.append(['DTYPE',self.name,1]) #Set it as output
			if self.type == 'OC0':
				self.Q.append(['WRITE','TCCR0',105])
			elif self.type == 'OC2':
				self.Q.append(['WRITE','TCCR2',105])
				self.Q.append(['WRITE','TCNT2',0])
			elif self.type == 'OC1A':
				self.Q.append(['WRITE','TCCR1A',131])
				self.Q.append(['WRITE','TCCR1B',1])


		if self.currentPage == 0: #Input
			self.Q.append(['DTYPE',self.name,0])
		elif self.currentPage == 1: #Output
			self.Q.append(['DTYPE',self.name,1])

	def setOutputState(self,state):
		self.nameOut.setChecked(state)
		self.pullup.setChecked(state)
		self.Q.append(['DSTATE',self.name,state])

	def setpwm(self,val):
		if self.type == 'OC0':
			self.Q.append(['WRITE','OCR0',val]) 
		elif self.type == 'OC2':
			self.Q.append(['WRITE','OCR2',val]) 
		elif self.type == 'OC1A':
			self.Q.append(['WRITE','OCR1AH',(val>>8)&0x3]) 
			self.Q.append(['WRITE','OCR1AL',(val)&0xFF]) 


class DIOADCCONFIG(QtWidgets.QDialog,ui_dio_adcLog.Ui_Dialog):
	def __init__(self,parent,name,opts,logstate,accepted):
		super(DIOADCCONFIG, self).__init__(parent)
		self.onFinished = accepted
		self.setupUi(self)
		self.setWindowTitle('Configure ADC Pin : %s'%parent.name)
		self.label.setText(name)
		self.scale = 1.
		self.log.setChecked(logstate)
		self.options.addItems(opts)
		self.gauge.update_value(200)
		self.gauge.set_MaxValue(1023)
		self.currentPage = 0
		self.graph.setRange(xRange=[-300, 0])
		self.curve = self.graph.plot(pen=colors[0])
		self.datapoints=0
		self.curvedata = np.empty(300)

	def changeRange(self,state):
		self.scale = 5000./1023. if state else 1.
		self.gauge.set_MaxValue(5000. if state else 1023)

	def setValue(self,val):
		if self.currentPage == 0: #Update Analog Gauges
			self.gauge.update_value(val*self.scale)
		elif self.currentPage == 1: #Update Data Logger
			self.curvedata[self.datapoints] = val * self.scale
			self.datapoints += 1 #Increment datapoints once per set. it's shared
			if self.datapoints >= self.curvedata.shape[0]-1:
				tmp = self.curvedata
				self.curvedata = np.empty(self.curvedata.shape[0] * 2) #double the size
				self.curvedata[:tmp.shape[0]] = tmp
			self.curve.setData(self.curvedata[:self.datapoints])
			self.curve.setPos(-self.datapoints, 0)





	def update(self):
		self.onFinished(self.options.currentText(),self.log.isChecked())

	def config(self,val):
		self.onFinished(self.options.currentText(),self.log.isChecked())

	def accept(self):
		self.onFinished(self.options.currentText(),self.log.isChecked())
		self.close()

	def next(self):
		if self.currentPage==1:
			self.currentPage = 0
			self.switcher.setText("Data Logger")
		else:
			self.currentPage = 1
			self.switcher.setText("Analog Gauge")

		self.monitors.setCurrentIndex(self.currentPage)


class DIOADC(QtWidgets.QStackedWidget,ui_dio_adc.Ui_stack):
	def __init__(self,name,Q,**kwargs):
		super(DIOADC, self).__init__()
		self.setupUi(self)
		self.name = name
		self.configWindow = None
		self.chan = int(self.name[2])
		self.logstate = False
		self.ADMUX = 64|self.chan #  AREF With Capacitor | PA[x]
		self.muxOptions = {
			self.name:self.chan,
			'ADC1-ADC0 @10x':0b01001,
			'ADC1-ADC0 @200x':0b01011,
			'ADC3-ADC2 @10x':0b01101,
			'ADC3-ADC2 @200x':0b01111,
			self.name+'-ADC1 @1x':0b10000|self.chan,
			'1.22V(BandGap Reference)':0b11110,
			'0V (Ground)':0b11111,
		}
		self.Q = Q #Command Queue
		self.nameOut.setText(name)
		self.nameIn.setText(name)
		self.nameIn.setEnabled(False)
		self.currentPage = 0
		self.lcdNumber.mousePressEvent = self.config

	def config(self,evt):
		if not self.configWindow:
			self.configWindow = DIOADCCONFIG(self,'ADMUX',self.muxOptions.keys(),self.logstate,self.setConfig)
		#self.configWindow.exec_() #Blocks UI (Modal), and only one instance can be shown
		self.configWindow.show() # Non blocking. Multiple.

	def setConfig(self,val,log):
		self.ADMUX = 64|self.muxOptions.get(val,self.chan)
		self.logstate = log


	def initPage(self):
		if self.currentPage == 0 or self.currentPage == 2: #Input or ADC
			self.Q.append(['DTYPE',self.name,0])
		elif self.currentPage == 1: #Output
			self.Q.append(['DTYPE',self.name,1])

	def setOutputState(self,state):
		self.nameOut.setChecked(state)
		self.pullup.setChecked(state)
		self.Q.append(['DSTATE',self.name,state])

	def setValue(self,val):
		self.slider.setValue(val)
		if self.configWindow:
			self.configWindow.setValue(val)

	def next(self):
		self.currentPage+=1
		if self.currentPage >= self.count():
			self.currentPage = 0
		self.setCurrentIndex(self.currentPage)
		self.initPage()
		
class DIOCNTR(QtWidgets.QFrame,ui_dio_cntr.Ui_Frame):
	def __init__(self,name,Q,**kwargs):
		super(DIOCNTR, self).__init__()
		self.setupUi(self)
		self.name = name
		
		self.Q = Q #Command Queue
		self.nameOut.setText(name)
		self.nameIn.setText(name)
		self.nameIn.setEnabled(False)
		self.currentPage = 0
		
	def setPage(self,page):
		self.currentPage=page
		self.stack.setCurrentIndex(self.currentPage)
		self.initPage()

	def setThreshold(self,val):
			self.thresLabel.setText(str(val)+u'\u21c6')
			self.Q.append(['WRITE','OCR1AH',(val>>8)&0xFF]) 
			self.Q.append(['WRITE','OCR1AL',(val)&0xFF]) 

	def initPage(self):
		if self.currentPage == 0 or self.currentPage == 2: #Input or CNTR INPUT
			self.Q.append(['DTYPE',self.name,0])
		elif self.currentPage == 1: #Output
			self.Q.append(['DTYPE',self.name,1])

		if self.currentPage == 2: #CNTR Input
				self.Q.append(['DTYPE','PD5',1])   #Set PD0 to output
				self.Q.append(['WRITE','TCNT1H',0]) # 
				self.Q.append(['WRITE','TCNT1L',0]) # 
				self.setThreshold(self.thresholdSlider.value())
				self.Q.append(['WRITE','TCCR1B',7]) # 0b111 - External clock input on T1
				self.Q.append(['WRITE','TCCR1A',64|4]) #Toggle PD5
				self.thresLabel.setText(u'PD5\u21c6')


	def setOutputState(self,state):
		self.nameOut.setChecked(state)
		self.pullup.setChecked(state)
		self.Q.append(['DSTATE',self.name,state])

class REGEDIT(QtWidgets.QFrame,ui_regedit.Ui_Frame):
	def __init__(self,Q):
		super(REGEDIT, self).__init__()
		self.setupUi(self)
		self.Q = Q
		self.regName.addItems([k for k in REGISTERS.VERSIONS[99]['REGISTERS'].keys() if k not in REGISTERS.VERSIONS[99]['RESTRICTED_REGISTERS']])

		self.type = 0 # 0=read. 1 =write
		self.bits = [self.b0,self.b1,self.b2,self.b3,self.b4,self.b5,self.b6,self.b7]
		for a in self.bits:
			a.clicked.connect(self.valueRefresh)
			a.setEnabled(self.type)

		self.labels = [self.l0,self.l1,self.l2,self.l3,self.l4,self.l5,self.l6,self.l7]
		self.format = '{0:d}'
		self.valueLabel.mousePressEvent = self.valueMouseClick
		self.typeLabel.mousePressEvent = self.changeType

	def execute(self):
		if self.type: #In write mode
			self.Q.append(['WRITE',str(self.regName.currentText()),self.getValue()])
		else:
			self.Q.append(['READ',str(self.regName.currentText()),self.setValue])

	def changeType(self,event):
		if self.type: #Was in write mode
			self.type = 0 #Change to read mode
			self.typeLabel.setText(u'READ\u2191')
		else:
			self.type = 1 #Change to write mode
			self.typeLabel.setText(u'WRITE\u2193')
		for a in self.bits:
			a.setEnabled(self.type)

	def getValue(self):
		val = 0
		for a in reversed(self.bits):
			val = val<<1
			if a.isChecked(): val|=1
		return val

	def setValue(self,val):
		for a in self.bits:
			a.setChecked(val&1)
			val =val>>1

	def valueMouseClick(self, event):
		if self.format == '{0:d}':
			self.format = '{0:08b}'
		elif self.format == '{0:08b}':
			self.format = '0x{0:02X}'
		elif self.format == '0x{0:02X}':
			self.format = '{0:d}'
		self.valueRefresh()

	def valueRefresh(self):
		self.valueLabel.setText(self.format.format(self.getValue()))


class DIOSENSOR(QtWidgets.QDialog,ui_dio_sensor.Ui_Dialog):
	def __init__(self,parent,sensor):
		super(DIOSENSOR, self).__init__(parent)
		name = sensor['name']
		self.initialize = sensor['init']
		self.read = sensor['read']
		self.isPaused = False
		self.setupUi(self)
		self.currentPage = 0
		self.scale = 1.
		self.max = sensor.get('max',None)
		self.min = sensor.get('min',None)
		self.fields = sensor.get('fields',None)
		self.RGB = sensor.get('RGB',False)
		self.widgets =[]
		for a in sensor.get('config',[]): #Load configuration menus
			l = QtWidgets.QLabel(a.get('name',''))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			l = QtWidgets.QComboBox(); l.addItems(a.get('options',[]))
			l.currentIndexChanged['int'].connect(a.get('function',None))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			
		self.graph.setRange(xRange=[-5, 0])
		import pyqtgraph as pg
		self.region = pg.LinearRegionItem()
		self.region.setBrush([255,0,50,50])
		self.region.setZValue(10)
		for a in self.region.lines: a.setCursor(QtGui.QCursor(QtCore.Qt.SizeHorCursor)); 
		self.graph.addItem(self.region, ignoreBounds=False)
		self.region.setRegion([-3,-.5])



		self.curves = {}; self.fitCurves = {}
		self.gauges = {}
		self.datapoints=0
		self.T = 0
		self.time = np.empty(300)
		self.start_time = time.time()
		row = 1; col=1;
		for a,b,c in zip(self.fields,self.min,self.max):
			gauge = Gauge(self)
			gauge.setObjectName(a)
			gauge.set_MinValue(b)
			gauge.set_MaxValue(c)
			#listItem = QtWidgets.QListWidgetItem()
			#self.listWidget.addItem(listItem)
			#self.listWidget.setItemWidget(listItem, gauge)
			self.gaugeLayout.addWidget(gauge,row,col)
			col+= 1
			if col == 4:
				row += 1
				col = 1
			self.gauges[gauge] = [a,b,c] #Name ,min, max value
			
			curve = self.graph.plot(pen=colors[len(self.curves.keys())])
			fitcurve = self.graph.plot(pen=colors[len(self.curves.keys())],width=2)
			self.curves[curve] = np.empty(300)
			self.fitCurves[curve] = fitcurve

		
		self.setWindowTitle('Sensor : %s'%name)

	def next(self):
		if self.currentPage==1:
			self.currentPage = 0
			self.switcher.setText("Data Logger")
		else:
			self.currentPage = 1
			self.switcher.setText("Analog Gauge")

		self.monitors.setCurrentIndex(self.currentPage)

	def changeRange(self,state):
		for a in self.gauges:
			self.scale = self.gauges[a][1]/65535. if state else 1.
			a.set_MaxValue(self.gauges[a][1] if state else 65535)

	def setValue(self,vals):
		if vals is None:
			print('check connections')
			return
		if self.currentPage == 0: #Update Analog Gauges
			p=0
			for a in self.gauges:
				a.update_value(vals[p]*self.scale)
				p+=1
		elif self.currentPage == 1: #Update Data Logger
			if self.isPaused: return
			p=0
			self.T = time.time() - self.start_time
			self.time[self.datapoints] = self.T
			if self.datapoints >= self.time.shape[0]-1:
				tmp = self.time
				self.time = np.empty(self.time.shape[0] * 2) #double the size
				self.time[:tmp.shape[0]] = tmp

			for a in self.curves:
				self.curves[a][self.datapoints] = vals[p] * self.scale
				if not p: self.datapoints += 1 #Increment datapoints once per set. it's shared

				if self.datapoints >= self.curves[a].shape[0]-1:
					tmp = self.curves[a]
					self.curves[a] = np.empty(self.curves[a].shape[0] * 2) #double the size
					self.curves[a][:tmp.shape[0]] = tmp
				a.setData(self.time[:self.datapoints],self.curves[a][:self.datapoints])
				a.setPos(-self.T, 0)
				p+=1
		if self.RGB:
			if len(vals)>=3:
				S = sum(vals[:3])
				R = int(255*vals[0]/S)
				G = int(255*vals[1]/S)
				B = int(255*vals[2]/S)
				self.page.setStyleSheet('background-color:rgb(%d,%d,%d);'%(R,G,B))
	def sineFit(self):
		self.pauseBox.setChecked(True)
		S,E=self.region.getRegion()
		start = (np.abs(self.time[:self.datapoints]- self.T - S)).argmin()
		end = (np.abs(self.time[:self.datapoints]-self.T - E)).argmin()
		print(self.T,start,end,S,E,self.time[start],self.time[end])
		res = ''
		for a in self.curves:
			try:
					fa=fit_sine(self.time[start:end],self.curves[a][start:end])
					if fa is not None:
							amp=abs(fa[0])
							freq=fa[1]
							s = '%5.2f , %5.3f Hz<br>'%(amp,freq)
							res+= s
							x = np.linspace(self.time[start],self.time[end],1000)
							self.fitCurves[a].clear()
							self.fitCurves[a].setData(x-self.T,sine_eval(x,fa))
			except Exception as e:
					res+='--<br>'
					print (e.message)
					pass
		QtWidgets.QMessageBox.information(self, ' Sine Fit Results ', res)


	def dampedSineFit(self):
		self.pauseBox.setChecked(True)
		S,E=self.region.getRegion()
		start = (np.abs(self.time[:self.datapoints]- self.T - S)).argmin()
		end = (np.abs(self.time[:self.datapoints]-self.T - E)).argmin()
		print(self.T,start,end,S,E,self.time[start],self.time[end])
		res = ''
		for a in self.curves:
			try:
					fa=fit_dsine(self.time[start:end],self.curves[a][start:end])
					if fa is not None:
							amp=abs(fa[0])
							freq=fa[1]
							decay=fa[4]
							s = '%5.2f , %5.3f Hz, %.3e<br>'%(amp,freq,decay)
							res+= s
							x = np.linspace(self.time[start],self.time[end],1000)
							self.fitCurves[a].clear()
							self.fitCurves[a].setData(x-self.T,dsine_eval(x,fa))
			except Exception as e:
					res+='--<br>'
					print (e.message)
					pass
		QtWidgets.QMessageBox.information(self, ' Sine Fit Results ', res)


	def pause(self,val):
		self.isPaused = val
		if not val: #clear fit plots
			for a in self.curves:
				self.fitCurves[a].clear()

	def launch(self):
		if self.initialize is not None:
			self.initialize()
		self.show()


class DIOCONTROL(QtWidgets.QDialog,ui_dio_control.Ui_Dialog):
	def __init__(self,parent,sensor):
		super(DIOCONTROL, self).__init__(parent)
		name = sensor['name']
		self.initialize = sensor['init']
		self.setupUi(self)
		self.widgets =[]
		self.gauges = {}
		self.functions = {}

		for a in sensor.get('write',[]): #Load configuration menus
			l = QtWidgets.QSlider(self); l.setMinimum(a[1]); l.setMaximum(a[2]);l.setValue(a[3]);
			l.setOrientation(QtCore.Qt.Horizontal)
			l.valueChanged['int'].connect(partial(self.write,l))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			
			gauge = Gauge(self)
			gauge.setObjectName(a[0])
			gauge.set_MinValue(a[1])
			gauge.set_MaxValue(a[2])
			gauge.update_value(a[3])
			self.gaugeLayout.addWidget(gauge)
			self.gauges[l] = gauge #Name ,min, max value,default value, func
			self.functions[l] = a[4]
			
		self.setWindowTitle('Control : %s'%name)

	def write(self,w,val):
		self.gauges[w].update_value(val)
		self.functions[w](val)


	def launch(self):
		self.initialize()
		self.show()




class DIOROBOT(QtWidgets.QDialog,ui_dio_robot.Ui_Dialog):
	def __init__(self,parent,sensor):
		super(DIOROBOT, self).__init__(parent)
		name = sensor['name']
		self.initialize = sensor['init']
		self.setupUi(self)
		self.widgets =[]
		self.gauges = OrderedDict()
		self.lastPos = OrderedDict()
		self.functions = OrderedDict()
		self.positions = []

		for a in sensor.get('write',[]): #Load configuration menus
			l = QtWidgets.QSlider(self); l.setMinimum(a[1]); l.setMaximum(a[2]);l.setValue(a[3]);
			l.setOrientation(QtCore.Qt.Horizontal)
			l.valueChanged['int'].connect(partial(self.write,l))
			self.configLayout.addWidget(l) ; self.widgets.append(l)
			
			gauge = Gauge(self)
			gauge.setObjectName(a[0])
			gauge.set_MinValue(a[1])
			gauge.set_MaxValue(a[2])
			gauge.update_value(a[3])
			self.lastPos[l] = a[3]
			self.gaugeLayout.addWidget(gauge)
			self.gauges[l] = gauge #Name ,min, max value,default value, func
			self.functions[l] = a[4]
			
		self.setWindowTitle('Control : %s'%name)

	def write(self,w,val):
		self.gauges[w].update_value(val)
		self.lastPos[w] = val
		self.functions[w](val)

	def add(self):
		self.positions.append([a.value() for a in self.lastPos.keys()])
		item = QtWidgets.QListWidgetItem("%s" % str(self.positions[-1]))
		self.listWidget.addItem(item)
		print(self.positions)

	def play(self):
		mypos = [a.value() for a in self.lastPos.keys()] # Current position
		sliders = list(self.gauges.keys())
		for nextpos in self.positions:
			dx = [(x-y) for x,y in zip(nextpos,mypos)]  #difference between next position and current
			distance = max(dx)
			for travel in range(20):
				for step in range(4):
						self.write(sliders[step],int(mypos[step]))
						mypos[step] += dx[step]/20.
				time.sleep(0.01)
							
						

	def launch(self):
		self.initialize()
		self.show()







class DIOSTEPPER(QtWidgets.QDialog,ui_dio_stepper.Ui_Dialog):
	def __init__(self,parent,**configuration):
		super(DIOSTEPPER, self).__init__(parent)
		name = 'stepper'
		self.setupUi(self)
		self.widgets =[]
		self.position = 0
		self.targetPosition = 0
		self.positions=[]
		self.lastStep = 0
		self.steps = [3,6,12,9]
		self.mask = 0xF0

		self.p = configuration.get('device',None)
		self.p.setReg('DDRB',255)

		self.gauge = Gauge(self)
		self.gauge.setObjectName("motor")
		self.gauge.set_MinValue(0)
		self.gauge.set_start_scale_angle(90)
		self.gauge.set_total_scale_angle_size(360)
		self.totalSteps = configuration.get('total',0)
		self.gauge.set_MaxValue(self.totalSteps)
		self.gauge.update_value(0)
		self.gaugeLayout.addWidget(self.gauge)
			
		self.setWindowTitle('Stepper Motor Control')

	def setPins(self,v):
		self.steps = [[3,6,12,9],[3<<4,6<<4,12<<4,9<<4]][v]
		self.mask = [0xF0,0x0F][v] #For ANDING 

	def initialize(self):
		self.targetPosition = 0		
	def stepLeft(self):
		self.targetPosition -= 1
	def stepRight(self):
		self.targetPosition += 1
	def stepTo(self):
		self.targetPosition = int(self.currentPositionBox.value())

	def read(self): #Read is not read. it actually updates the motor position.
		if self.position==self.targetPosition:
			return None #Could return position, but that disables needle dragging functionality of the gauge

		if self.position > self.targetPosition:
			self.lastStep -= 1
			if self.lastStep == -1: self.lastStep = 3
			self.position-=1
		elif self.position < self.targetPosition:
			self.lastStep += 1
			if self.lastStep == 4: self.lastStep = 0
			self.position+=1
		prev = self.p.REGSTATES['PORTB']
		self.p.setReg('PORTB',prev&self.mask|self.steps[self.lastStep])
		return self.position

	def setValue(self,val):
		if val is not None:
			self.gauge.update_value(val)

	def add(self):
		self.positions.append(self.position)
		item = QtWidgets.QListWidgetItem("%s" % self.position)
		self.listWidget.addItem(item)

	def play(self):
		pass

	def launch(self):
		self.initialize()
		self.show()



'''
For Malus Law experiment
Connect stepper motor to PB0, PB1, PB2, PB3 (A+,B+,A-,B-)
Connect TSL2561 sensor to I2C
LASER Beam -> Polarizer -> Analyzer mounted on stepper motor -> TSL2561
'''
#from kuttyPy import *
from matplotlib import pyplot as plt
TSL2561_init()
TSL2561_gain(1)
TSL2561_timing(0)
setReg('DDRB',15)
steps=[3,6,12,9]
points = 0
Y=[]
for a in range(50):
	for b in steps:
		setReg('PORTB',b)
		plt.pause(0.1)
		lux = TSL2561_all()
		if lux is not None:
			print(points,lux)
			plt.scatter(points,lux[0])
			points+=1
		
	



from ..Qt import QtGui,QtCore,QtWidgets

class Guage(QtWidgets.QWidget):
	valueChanged = QtCore.pyqtSignal(float)
	def __init__(self, parent=None):
		super(Guage, self).__init__(parent)
		self.setWindowTitle("Analog Clock")
		self.L=285.
		self.resize(self.L, self.L)
		self.hand = QtGui.QPolygon([
			QtCore.QPoint(7, 8),
			QtCore.QPoint(-7, 8),
			QtCore.QPoint(0, -80)
		])
		self.color = QtGui.QColor(40, 127, 57, 210)
		self.mcolor = QtGui.QColor(0, 255, 127, 191)
		self.min = 0.
		self.max = 100.
		self.range = self.max - self.min
		self.value = 100.

	def paintEvent(self, event):
		side = min(self.width(), self.height())

		painter = QtGui.QPainter()
		painter.begin(self)
		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		painter.translate(self.width() / 2, self.height() / 2)
		painter.scale(side / self.L, side / self.L)

		painter.setPen(QtCore.Qt.NoPen)
		painter.setBrush(QtGui.QBrush(self.color))

		painter.save()
		painter.rotate(250*(self.value/self.range)-125)
		painter.drawConvexPolygon(self.hand)
		painter.restore()

		painter.setPen(self.color)
		txt = str(self.value)
		painter.setFont(QtGui.QFont("Helvetica", 30))
		painter.drawText(QtCore.QPoint(-10*len(txt), 95), txt)

		painter.rotate(145)
		for i in range(0, 11):
			painter.drawLine(85, 0, 96, 0)
			painter.rotate(25.0)

		painter.setPen(QtGui.QPen(self.mcolor))
		painter.rotate(85)
		for i in range(0, 51):
			painter.drawLine(92, 0, 96, 0)
			painter.rotate(5.0)

		painter.setFont(QtGui.QFont("Helvetica", 10))
		painter.setPen(QtGui.QPen(self.color))
		painter.rotate(-125)
		for i in range(5, 11):
			painter.drawText(QtCore.QPoint(97, 0), str(self.range*i/10));
			painter.rotate(25)

		painter.rotate(260)
		for i in range(5):
			txt = str(self.range*i/10)
			painter.drawText(QtCore.QPoint(-90-len(txt)*10, 0), txt);
			painter.rotate(25)
		painter.end()

	def minimumSizeHint(self):
		return QtCore.QSize(50, 50)

	def sizeHint(self):
		return QtCore.QSize(100, 100)

	def updateValue(self,val):
		self.valueChanged.emit(val)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'regvals.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(288, 31)
        Frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Frame)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.L1 = QtWidgets.QLabel(Frame)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.L1.setFont(font)
        self.L1.setAlignment(QtCore.Qt.AlignCenter)
        self.L1.setObjectName("L1")
        self.horizontalLayout.addWidget(self.L1)
        self.L2 = QtWidgets.QLabel(Frame)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.L2.setFont(font)
        self.L2.setAlignment(QtCore.Qt.AlignCenter)
        self.L2.setObjectName("L2")
        self.horizontalLayout.addWidget(self.L2)
        self.L3 = QtWidgets.QLabel(Frame)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.L3.setFont(font)
        self.L3.setAlignment(QtCore.Qt.AlignCenter)
        self.L3.setObjectName("L3")
        self.horizontalLayout.addWidget(self.L3)

        self.retranslateUi(Frame)
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        _translate = QtCore.QCoreApplication.translate
        Frame.setWindowTitle(_translate("Frame", "Frame"))
        self.L1.setText(_translate("Frame", "DDR\n"
"-"))
        self.L2.setText(_translate("Frame", "PORT\n"
"-"))
        self.L3.setText(_translate("Frame", "PIN\n"
"-"))



# Read ADC Values and enable a relay if the value exceeds
# the midpoint value of 512.
# This simple demo can be used to make a thermostat

import time
#from kuttyPy import *
setReg('DDRC',1)
setReg('ADMUX', (1<<6) | 5) #REF_AVCC | Channel 5
for a in range(50): 
    setReg('ADCSRA', 196)
    cl = getReg('ADCL')
    ch = getReg('ADCH')
    val = (ch<<8)|cl
    if(val>512):setReg('PORTC',1)
    else: setReg('PORTC',0)
    print(val)
    time.sleep(0.1)
    



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ipy.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(847, 411)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.layout = QtWidgets.QVBoxLayout(self.frame)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        self.layout.setObjectName("layout")
        self.verticalLayout.addWidget(self.frame)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBar.sizePolicy().hasHeightForWidth())
        self.toolBar.setSizePolicy(sizePolicy)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionNumpy = QtWidgets.QAction(MainWindow)
        self.actionNumpy.setObjectName("actionNumpy")
        self.actionPylab = QtWidgets.QAction(MainWindow)
        self.actionPylab.setObjectName("actionPylab")
        self.actionScipy = QtWidgets.QAction(MainWindow)
        self.actionScipy.setObjectName("actionScipy")
        self.toolBar.addAction(self.actionNumpy)
        self.toolBar.addAction(self.actionScipy)
        self.toolBar.addAction(self.actionPylab)

        self.retranslateUi(MainWindow)
        self.actionNumpy.triggered.connect(MainWindow.importNumpy) # type: ignore
        self.actionScipy.triggered.connect(MainWindow.importScipy) # type: ignore
        self.actionPylab.triggered.connect(MainWindow.importPylab) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "iPython Console"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.actionNumpy.setText(_translate("MainWindow", "Numpy"))
        self.actionNumpy.setToolTip(_translate("MainWindow", "import numpy as np"))
        self.actionNumpy.setShortcut(_translate("MainWindow", "Ctrl+Shift+N"))
        self.actionPylab.setText(_translate("MainWindow", "PyLab"))
        self.actionPylab.setToolTip(_translate("MainWindow", "from pylab import *"))
        self.actionPylab.setShortcut(_translate("MainWindow", "Ctrl+Shift+P"))
        self.actionScipy.setText(_translate("MainWindow", "Scipy"))
        self.actionScipy.setToolTip(_translate("MainWindow", "import scipy"))
        self.actionScipy.setShortcut(_translate("MainWindow", "Ctrl+Shift+S"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio_adcConfig.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(395, 298)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.log_2 = QtWidgets.QCheckBox(Dialog)
        self.log_2.setObjectName("log_2")
        self.gridLayout.addWidget(self.log_2, 1, 2, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 4, 0, 1, 3)
        self.log = QtWidgets.QCheckBox(Dialog)
        self.log.setObjectName("log")
        self.gridLayout.addWidget(self.log, 1, 0, 1, 1)
        self.gauge = Gauge(Dialog)
        self.gauge.setObjectName("gauge")
        self.gridLayout.addWidget(self.gauge, 3, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 1, 1, 1, 1)
        self.options = QtWidgets.QComboBox(Dialog)
        self.options.setObjectName("options")
        self.gridLayout.addWidget(self.options, 0, 1, 1, 2)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        self.log_2.toggled['bool'].connect(Dialog.changeRange) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "ADC Configuration"))
        self.log_2.setText(_translate("Dialog", "0-5000mV Range"))
        self.label.setText(_translate("Dialog", "ADMUX"))
        self.log.setText(_translate("Dialog", "Log Read/Write calls"))
from .gauge import Gauge



import time
#from kuttyPy import *
setReg('DDRB',255)

while 1:
	for x in range(20):
		for a in [3,6,12,9]:    
			setReg('PORTB', a)
			time.sleep(0.1)

	for x in range(20):
		for a in [12,6,3,9]:    
			setReg('PORTB', a)
			time.sleep(0.1)

setReg('PORTB', 0)



'''
'''
# !/usr/bin/python3

import os, sys, time, re, traceback, platform
from PyQt5 import QtGui, QtCore, QtWidgets
import KuttyPyLib
import socket

from utilities.templates import ui_layout_ide as layout
from utilities import uploader, syntax, REGISTERS, dio
from utilities import texteditor
import constants
import inspect

from functools import partial
from collections import OrderedDict


# translation stuff
def translate(lang=None):
	global app, t, t1
	if lang is None:
		lang = QtCore.QLocale.system().name()
	t = QtCore.QTranslator()
	t.load("lang/" + lang, os.path.dirname(__file__))
	app.installTranslator(t)
	t1 = QtCore.QTranslator()
	t1.load("qt_" + lang,
	        QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	app.installTranslator(t1)


class myTimer():
	def __init__(self, interval):
		self.interval = interval
		self.reset()

	def reset(self):
		self.timeout = time.time() + self.interval

	def ready(self):
		T = time.time()
		dt = T - self.timeout
		if dt > 0:  # timeout is ahead of current time
			# if self.interval>5:print('reset',self.timeout,dt)
			self.timeout = T - dt % self.interval + self.interval
			# if self.interval>5:print(self.timeout)
			return True
		return False

	def progress(self):
		return 100 * (self.interval - self.timeout + time.time()) / (self.interval)


LKP = True


class AppWindow(QtWidgets.QMainWindow, layout.Ui_MainWindow):
	p = None
	logThis = QtCore.pyqtSignal(str)
	showStatusSignal = QtCore.pyqtSignal(str, bool)
	serverSignal = QtCore.pyqtSignal(str)
	logThisPlain = QtCore.pyqtSignal(bytes)
	codeOutput = QtCore.pyqtSignal(str, str)
	serialGaugeSignal = QtCore.pyqtSignal(bytes)
	serialGaugeConvert = 'bytes'
	serialStream = b''

	def __init__(self, parent=None, **kwargs):
		super(AppWindow, self).__init__(parent)

		self.local_ip = ''
		self.setupUi(self)

		self.fs_watcher = None
		self.reloadFrame.setVisible(False)

		self.compile_thread = None

		self.monitoring = True
		self.userHexRunning = False
		self.uploadingHex = False
		self.autoUpdateUserRegisters = False
		self.CFile = None  # '~/kuttyPy.c'
		self.defaultDirectory = path["examples"]
		self.serverActive = False
		self.VERSION = REGISTERS.VERSION_ATMEGA32  # This needs to be dynamically changed when hardware is connected

		self.serialFrame.hide()  # Hide the serial guage button
		# Define some keyboard shortcuts for ease of use
		self.shortcutActions = {}
		self.shortcuts = {"f": partial(self.setLanguage, 'fr_FR'), "e": partial(self.setLanguage, 'en_IN'),
		                  "m": partial(self.setLanguage, 'ml_IN')}
		for a in self.shortcuts:
			shortcut = QtWidgets.QShortcut(QtGui.QKeySequence(a), self)
			shortcut.activated.connect(self.shortcuts[a])
			self.shortcutActions[a] = shortcut

		########## C CODE EDITOR SYNTAX HIGHLIGHTER
		self.code_highlighters = []
		####### C CODE EDITOR #########
		self.codingTabs.tabCloseRequested.connect(self.closeCTab)
		self.codingTabs.tabBarClicked.connect(self.CTabChanged)
		self.listTab = None
		self.mapTab = None
		self.hexTab = None
		self.serialGaugeSignal.connect(self.setSerialgauge)
		self.logThisPlain.connect(self.appendLogPlain)  # Connect to the log window

		self.activeEditor = None
		self.activeSourceTab = None
		self.sourceTabs = {}
		self.addSourceTab()

		######### C CODE UPLOADER
		self.uploadThread = QtCore.QThread()
		self.UploadObject = self.uploadObject()
		self.UploadObject.moveToThread(self.uploadThread)
		self.UploadObject.finished.connect(self.uploadThread.quit)
		self.UploadObject.logThis.connect(self.appendLog)  # Connect to the log window
		self.UploadObject.resultSignal.connect(self.codeOutput)  # Load .lst, .hex , .map files
		self.UploadObject.logThisPlain.connect(self.appendLogPlain)  # Connect to the log window. add plain text
		self.logThis.connect(self.appendLog)  # Connect to the log window

		self.uploadThread.started.connect(self.UploadObject.execute)
		self.uploadThread.finished.connect(self.codeFinished)

		self.statusBar = self.statusBar()
		self.makeBottomMenu()
		self.addFileMenu()
		self.addEditMenu()
		self.addBuildOptionsMenu()

		self.editorFont = QtGui.QFont()
		self.editorFont.setPointSize(12)
		self.editorFont.setFamily('Ubuntu mono')

		global app

		self.initializeCommunications()
		self.pending = {
			'status': myTimer(constants.STATUS_UPDATE_INTERVAL),
			'update': myTimer(constants.AUTOUPDATE_INTERVAL),
		}

		serialgaugeoptions = {'name': 'Serial Monitor', 'init': print, 'read': None,
		                      'fields': ['Value'],
		                      'min': [0],
		                      'max': [1000],
		                      'config': [{
			                      'name': 'Data Type',
			                      'options': ['byte', 'ASCII'],
			                      'function': self.configSerialGauge
		                      }
		                      ]}
		self.serialGauge = dio.DIOSENSOR(self, serialgaugeoptions)

		self.startTime = time.time()
		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.updateEverything)
		self.timer.start(20)

		# Auto-Detector
		self.shortlist = KuttyPyLib.getFreePorts()

	def closeEvent(self, event):
		self.external.terminate()
		self.external.waitForFinished(1000)

	def embedTerminal(self):
		import subprocess
		system = platform.system()
		self.external = QtCore.QProcess(self)
		if system == 'Linux':
			self.external.start('gnome-terminal', ["--working-directory", self.defaultDirectory])
		if system == 'Windows':
			self.external.start('cmd')

		'''
		time.sleep(1)
		self.external.write(b"hello")

		p = subprocess.run(['xprop', '-root'], stdout=subprocess.PIPE)
		for line in p.stdout.decode().splitlines():
			m = re.fullmatch(r'^_NET_ACTIVE_WINDOW.*[)].*window id # (0x[0-9a-f]+)', line)
			if m:
				window = QtGui.QWindow.fromWinId(int(m.group(1), 16))
				window.setFlag(QtCore.Qt.FramelessWindowHint, True)
				widget = QtWidgets.QWidget.createWindowContainer(
					window, self.termFrame, QtCore.Qt.FramelessWindowHint)
				widget.setFixedSize(600, 400)
				self.termLayout.addWidget(widget)
				# this is where the magic happens...
				self.external.finished.connect(self.close_maybe)
				break
		else:
			QtWidgets.QMessageBox.warning(self, 'Error', 'Could not find WID for curreent Window')
		'''

	def close_maybe(self):
		print('terminal closed')
		pass

	def activateCompileServer(self):
		if self.serverActive:  # Stop it
			'''
			if self.compile_thread is not None:
				self.compile_thread.terminate()
				self.compile_thread.wait()
				print('quit compile_thread')
				self.compile_thread_button.setParent(None)
				self.showStatus("Compiler Stopped ", False)
				return
			self.serverActive = False
			self.compile_thread_button.setText("Online Compiler")
			'''
			return
		else:
			s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			s.settimeout(0.1)  # Set a timeout to avoid blocking indefinitely
			s.connect(("8.8.8.8", 80))  # Connect to a public IP address
			self.local_ip = s.getsockname()[0]

			from utilities.compile_server import create_server
			self.compile_thread = create_server(self.showStatusSignal, self.serverSignal, path["kpy"], self.local_ip)
			self.showStatusSignal.connect(self.showStatus)
			self.serverSignal.connect(self.showServerStatus)
			s.close()
			self.showStatus("Compiler Active at " + self.local_ip, False)
			self.compile_thread_button.setText(self.local_ip)
			self.serverActive = True

	def addFileMenu(self):
		codeMenu = QtWidgets.QMenu()

		newFileAction = QtWidgets.QAction('New File', self)
		newFileAction.setShortcut(QtGui.QKeySequence("Ctrl+N"))
		ico = QtGui.QIcon()
		ico.addPixmap(QtGui.QPixmap(":/control/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		newFileAction.setIcon(ico)
		newFileAction.triggered.connect(self.addSourceTab)
		codeMenu.addAction(newFileAction)

		openFileAction = QtWidgets.QAction('Open File', self)
		openFileAction.setShortcut(QtGui.QKeySequence("Ctrl+O"))
		openFileAction.triggered.connect(self.openFile)
		openIcon = QtGui.QIcon()
		openIcon.addPixmap(QtGui.QPixmap(":/control/document-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		openFileAction.setIcon(openIcon)
		codeMenu.addAction(openFileAction)

		saveFileAction = QtWidgets.QAction('Save File', self)
		saveFileAction.setShortcut(QtGui.QKeySequence("Ctrl+S"))
		saveFileAction.triggered.connect(self.saveFile)
		saveIcon = QtGui.QIcon()
		saveIcon.addPixmap(QtGui.QPixmap(":/control/saveall.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		saveFileAction.setIcon(saveIcon)
		codeMenu.addAction(saveFileAction)

		saveAsFileAction = QtWidgets.QAction('Save As', self)
		saveAsFileAction.setShortcut(QtGui.QKeySequence("Ctrl+Shift+S"))
		saveAsFileAction.triggered.connect(self.saveAs)
		saveAsFileAction.setIcon(saveIcon)
		codeMenu.addAction(saveAsFileAction)

		a = QtWidgets.QAction('Terminal', self)
		a.setShortcut(QtGui.QKeySequence("Ctrl+Shift+T"))
		a.triggered.connect(self.embedTerminal)
		termIcon = QtGui.QIcon()
		termIcon.addPixmap(QtGui.QPixmap(":/control/utilities-terminal.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		a.setIcon(termIcon)
		codeMenu.addAction(a)

		exitAction = QtWidgets.QAction('Exit', self)
		exitAction.triggered.connect(QtWidgets.qApp.quit)
		codeMenu.addAction(exitAction)
		self.fileMenuButton.setMenu(codeMenu)

	def addBuildOptionsMenu(self):
		codeMenu = QtWidgets.QMenu()
		toggleLKPAction = QtWidgets.QAction('Include KP Library', self)
		toggleLKPAction.setCheckable(True)
		toggleLKPAction.setChecked(True)
		toggleLKPAction.triggered[bool].connect(self.setLKP)
		codeMenu.addAction(toggleLKPAction)
		self.buildOptionsButton.setMenu(codeMenu)

	def setLKP(self, state):
		global LKP
		print('LKP linking', state)
		LKP = state

	def closeEvent(self, evnt):
		evnt.ignore()
		self.askBeforeQuit()

	def askBeforeQuit(self):
		ask = False
		for editors in self.sourceTabs:
			if self.sourceTabs[editors][0].changed:
				ask = True
		if ask:
			reply = QtWidgets.QMessageBox.question(self, 'Warning', 'Files may have unsaved changes.\nReally quit?',
			                                       QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
			if reply == QtWidgets.QMessageBox.No:
				return

		self.userHexRunning = False
		global app
		app.quit()

	def addEditMenu(self):
		codeMenu = QtWidgets.QMenu()

		undoAction = QtWidgets.QAction('Undo', self)
		undoAction.setShortcut(QtGui.QKeySequence("Ctrl+Z"))
		undoAction.triggered.connect(self.activeEditor.undo)
		ico = QtGui.QIcon()
		ico.addPixmap(QtGui.QPixmap(":/control/reset.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		undoAction.setIcon(ico)
		codeMenu.addAction(undoAction)

		redoAction = QtWidgets.QAction('Redo', self)
		redoAction.setShortcut(QtGui.QKeySequence("Ctrl+Shift+Z"))
		redoAction.triggered.connect(self.activeEditor.redo)
		codeMenu.addAction(redoAction)
		a = QtWidgets.QAction('Cut', self)
		a.setShortcut(QtGui.QKeySequence("Ctrl+X"))
		a.triggered.connect(self.activeEditor.cut)
		codeMenu.addAction(a)
		a = QtWidgets.QAction('Copy', self)
		a.setShortcut(QtGui.QKeySequence("Ctrl+C"))
		a.triggered.connect(self.activeEditor.copy)
		codeMenu.addAction(a)
		a = QtWidgets.QAction('Paste', self)
		a.setShortcut(QtGui.QKeySequence("Ctrl+V"))
		a.triggered.connect(self.activeEditor.paste)
		codeMenu.addAction(a)
		a = QtWidgets.QAction('Select All', self)
		a.setShortcut(QtGui.QKeySequence("Ctrl+A"))
		a.triggered.connect(self.activeEditor.selectAll)
		codeMenu.addAction(a)

		self.editMenuButton.setMenu(codeMenu)

	def makeBottomMenu(self):
		try:
			self.pushbutton.setParent(None)
		except:
			pass
		self.pushbutton = QtWidgets.QPushButton('Menu')
		self.pushbutton.setStyleSheet("color: #262;")
		menu = QtWidgets.QMenu()

		menu.addAction('Save Window as Svg', self.exportSvg)
		menu.addAction('Upload Hex File', self.uploadHex)

		# Theme
		self.themeAction = QtWidgets.QWidgetAction(menu)
		themes = [a.split('.qss')[0] for a in os.listdir(path["themes"]) if '.qss' in a]
		self.themeBox = QtWidgets.QComboBox();
		self.themeBox.addItems(themes)
		self.themeBox.currentIndexChanged['QString'].connect(self.setTheme)
		self.themeAction.setDefaultWidget(self.themeBox)
		menu.addAction(self.themeAction)

		defaultTheme = "newtheme"
		self.themeBox.setCurrentIndex(themes.index(defaultTheme))
		self.setTheme(defaultTheme)

		self.pushbutton.setMenu(menu)

		# Compile thread
		self.compile_thread_button = QtWidgets.QPushButton('Online Compiler')
		self.compile_thread_button.setStyleSheet("color: #262;")
		self.compile_thread_button.clicked.connect(self.activateCompileServer)
		self.statusBar.addPermanentWidget(self.compile_thread_button)

		self.bottomLabel = QtWidgets.QLabel("Messages")

		# Menu button
		self.statusBar.addPermanentWidget(self.pushbutton)

	def closeCTab(self, index):
		print('Close Tab', index)
		widget = self.codingTabs.widget(index)
		sourceTabClosed = False
		if widget == self.mapTab:
			print('closing map tab')
			self.mapTab = None
		elif widget == self.hexTab:
			print('closing hex tab')
			self.hexTab = None
		elif widget == self.listTab:
			print('closing list tab')
			self.listTab = None
		elif widget in self.sourceTabs:
			if len(self.sourceTabs) == 1:
				print("last tab. won't close.")
				return
			else:
				print('closing source tab', widget.objectName())
				self.closeCompileTabs(self.codingTabs.tabText(self.codingTabs.indexOf(widget)))
				self.sourceTabs.pop(widget)
				sourceTabClosed = True

		self.codingTabs.removeTab(index)
		if sourceTabClosed:  # Source Tab closed. Re-assign active source tab
			self.activeSourceTab = list(self.sourceTabs.keys())[0]
			self.activeEditor = self.sourceTabs[self.activeSourceTab][0]
			self.CFile = self.sourceTabs[self.activeSourceTab][1]
			self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(self.activeSourceTab))
			print('New Source Tab:', self.getActiveFilename(), self.CFile)

	def closeCompileTabs(self, fullname):
		name = os.path.split(fullname)[1].split('.')[0]
		for tab, ext in zip([self.mapTab, self.hexTab, self.listTab], ['.map', '.hex', '.lst']):
			if tab is not None:
				print(self.codingTabs.tabText(self.codingTabs.indexOf(tab)), name + ext)
				if self.codingTabs.tabText(self.codingTabs.indexOf(tab)) == name + ext:
					self.codingTabs.removeTab(self.codingTabs.indexOf(tab))
			# print(f"closing {ext} tab")

		pass

	def getActiveFilename(self):
		self.codingTabs.tabText(self.codingTabs.indexOf(self.activeSourceTab))

	def addSourceTab(self):
		sourceTab = QtWidgets.QWidget()
		sourceTab.setObjectName("sourceTab")
		horizontalLayout_3 = QtWidgets.QHBoxLayout(sourceTab)
		horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
		horizontalLayout_3.setSpacing(0)
		horizontalLayout_3.setObjectName("horizontalLayout_3")
		editor = texteditor.myTextEditor(sourceTab, self.codingTabs)
		font = QtGui.QFont()
		font.setFamily("Ubuntu Mono")
		font.setPointSize(12)
		editor.setFont(font)
		editor.setObjectName("editor")
		editor.setTabChangesFocus(False)
		horizontalLayout_3.addWidget(editor)
		self.codingTabs.addTab(sourceTab, "")
		self.sourceTabs[sourceTab] = [editor, None]
		self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(sourceTab))
		self.codingTabs.setTabText(self.codingTabs.indexOf(sourceTab), 'Untitled')
		self.CFile = None
		self.activeSourceTab = sourceTab
		self.activeEditor = editor
		self.code_highlighters.append(syntax.CHighlighter(editor.document()))

	def CTabChanged(self, index):
		widget = self.codingTabs.widget(index)
		if widget in self.sourceTabs:
			self.activeSourceTab = widget
			self.activeEditor = self.sourceTabs[widget][0]
			self.CFile = self.sourceTabs[widget][1]
			self.filenameLabel.setText(self.CFile)
			print('Change Tab', index, self.codingTabs.tabText(index), self.CFile)

	def newRegister(self):
		reg = dio.REGEDIT(self.commandQ)
		# self.registerLayout.addWidget(reg)
		# self.registers.append(reg)

		# TODO: Convert layout to listwidget to enable re-ordering
		regItem = QtWidgets.QListWidgetItem()
		regItem.setSizeHint(QtCore.QSize(200, 40))
		self.registerList.addItem(regItem)
		self.registerList.setItemWidget(regItem, reg)
		self.registers.append(regItem)

	def addPins(self):
		for port, dock in zip(self.ports, [self.palayout, self.pblayout, self.pclayout, self.pdlayout]):
			checkbox = dio.REGVALS(port)
			dock.addWidget(checkbox)
			self.btns[port] = checkbox

			seq = range(7, -1, -1)
			if port == 'C': seq = reversed(seq)  # PORTC pins are ordered top to bottom
			for a in seq:
				name = 'P' + port + str(a)
				checkbox = dio.widget(name, self.commandQ, extra=self.SPECIALS.get(name, ''))
				dock.addWidget(checkbox)
				self.btns[name] = checkbox

	def tabChanged(self, index):
		if self.userHexRunning:  # Firmware was running. Stop it.
			self.launchFirmwareButton.setChecked(False)
			self.jumpToApplication(False)

		if index != 0:  # examples/editor tab. disable monitoring
			self.monitoring = False
		else:  # Playground . enable monitoring and control.
			self.monitoring = True
			self.autoRefreshUserRegisters.setChecked(False)
			self.userRegistersAutoRefresh(False)
			self.setLogType('playground')

	def setLogType(self, tp):
		self.log.clear()
		if tp == 'playground':
			self.logLabel.setText("Monitor registers being read and set during each operation")
		elif tp == 'avr':
			self.logLabel.setText("Avr-Gcc compile/upload messages")
		elif tp == 'monitor':
			self.logLabel.setText("Monitor data input from the serial port")
		elif tp == 'python':
			self.logLabel.setText("Python Code operations monitor")

	################USER CODE SECTION####################
	def codeOutput(self, filetype, filename):
		self.openCodeBreakupTab(filetype, filename)
		try:
			infile = open(filename, 'r')
			if filetype == 'list':
				self.listEditor.setPlainText(infile.read())
			if filetype == 'map':
				self.mapEditor.setPlainText(infile.read())
			if filetype == 'hex':
				self.hexEditor.setPlainText(infile.read())
			infile.close()
		except Exception as e:
			print(e)

	def openCodeBreakupTab(self, filetype, filename):
		tab = None
		if filetype == 'list':
			if self.listTab is None:
				self.openListTab()
			elif self.listTab.isHidden():
				self.codingTabs.addTab(self.listTab, "")
			tab = self.listTab
		elif filetype == 'map':
			if self.mapTab is None:
				self.openMapTab()
			elif self.mapTab.isHidden():
				self.codingTabs.addTab(self.mapTab, "")
			tab = self.mapTab
		elif filetype == 'hex':
			if self.hexTab is None:
				self.openHexTab()
			elif self.hexTab.isHidden():
				self.codingTabs.addTab(self.hexTab, "")
			tab = self.hexTab

		if tab != None:
			fname = filename.split
			self.codingTabs.setTabText(self.codingTabs.indexOf(tab), os.path.basename(filename))  # filetype + ":" +

	def openListTab(self):
		self.listTab = QtWidgets.QWidget()
		self.listTab.setObjectName("listTab")
		self.verticalLayout = QtWidgets.QVBoxLayout(self.listTab)
		self.verticalLayout.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout.setSpacing(0)
		self.verticalLayout.setObjectName("verticalLayout")
		self.listEditor = QtWidgets.QTextBrowser(self.listTab)
		self.listEditor.setObjectName("listEditor")
		self.verticalLayout.addWidget(self.listEditor)
		self.codingTabs.addTab(self.listTab, "")

	def openMapTab(self):
		self.mapTab = QtWidgets.QWidget()
		self.mapTab.setObjectName("mapTab")
		self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.mapTab)
		self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout_3.setSpacing(0)
		self.verticalLayout_3.setObjectName("verticalLayout_3")
		self.mapEditor = QtWidgets.QTextBrowser(self.mapTab)
		self.mapEditor.setObjectName("mapEditor")
		self.verticalLayout_3.addWidget(self.mapEditor)
		self.codingTabs.addTab(self.mapTab, "")

	def openHexTab(self):
		self.hexTab = QtWidgets.QWidget()
		self.hexTab.setObjectName("hexTab")
		self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.hexTab)
		self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
		self.verticalLayout_2.setSpacing(0)
		self.verticalLayout_2.setObjectName("verticalLayout_2")
		self.hexEditor = QtWidgets.QTextBrowser(self.hexTab)
		self.hexEditor.setObjectName("hexEditor")
		self.verticalLayout_2.addWidget(self.hexEditor)
		self.codingTabs.addTab(self.hexTab, "")
		ico = QtGui.QIcon()
		ico.addPixmap(QtGui.QPixmap(":/control/hex.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.codingTabs.setTabIcon(self.codingTabs.indexOf(self.hexTab), ico)

	def appendLog(self, txt):
		self.log.append(txt)

	def appendLogPlain(self, txt):
		self.log.moveCursor(QtGui.QTextCursor.End)
		self.log.insertPlainText(txt.decode('ascii'))

	def configSerialGauge(self, val):
		if val == 0:  # 'byte'
			print('Byte Mode(0-255)')
			for a in self.serialGauge.gauges:
				a.set_MaxValue(255)
			self.serialGaugeConvert = 'bytes'
		if val == 1:  # 'ascii'
			print('ASCII(0-10000)')
			for a in self.serialGauge.gauges:
				a.set_MaxValue(10000)
			self.serialGaugeConvert = 'ascii'

	def setSerialgauge(self, vals):
		if self.serialGaugeConvert == 'bytes':
			self.serialGauge.setValue([vals[0]])
		elif self.serialGaugeConvert == 'ascii':
			self.serialStream += vals
			while b'\n' in self.serialStream:
				val, _, self.serialStream = self.serialStream.partition(b'\n')
				self.serialGauge.setValue([float(val)])

	def genLog(self):
		html = '''<table border="1" align="center" cellpadding="1" cellspacing="0" style="font-family:arial,helvetica,sans-serif;font-size:9pt;">
		<tbody><tr><td colspan="4">%s</td></tr>''' % (time.ctime())
		# html+='''<tr><td style="background-color:#77cfbb;">R/W</td><td style="background-color:#77cfbb;">REGISTER</td>
		# <td style="background-color:#77cfbb;">Value</td><td style="background-color:#77cfbb;">Hex/Binary</td></tr>'''

		for a in self.updatedRegs:
			row = self.updatedRegs[a]
			html += u'''
				<tr>
					<td>{0:s}</td>
					<td>{1:s}</td>
					<td>{2:d}</td>
					<td>0b{2:08b} | 0x{2:02x}</td>
				</tr>
				'''.format(u'W \u2193' if row[0] else u'R \u2191', a, row[1])
		html += "</tbody></table>"
		self.log.setHtml(html)

	def updateEverything(self):
		self.locateDevices()
		if not self.checkConnectionStatus(): return
		# KuttyPy monitor has handed over control to native code. act as serial monitor/ debug window
		if self.uploadingHex:
			return

		if self.userHexRunning:
			t = self.p.fd.read(self.p.fd.in_waiting)
			if len(t):
				self.serialGaugeSignal.emit(t)
				self.logThisPlain.emit(t)
			return

	########################### UPLOAD HEX FILE #######################

	class uploadObject(QtCore.QObject):
		finished = QtCore.pyqtSignal()
		logThis = QtCore.pyqtSignal(str)
		resultSignal = QtCore.pyqtSignal(str, str)
		logThisPlain = QtCore.pyqtSignal(bytes)
		fname = ''
		p = None

		def __init__(self):
			super(AppWindow.uploadObject, self).__init__()

		def config(self, mode, p, fname):
			self.p = p
			self.fname = fname
			self.mode = mode

		def execute(self):
			global LKP
			if 'compile' in self.mode:
				try:
					import subprocess
					if self.fname[-2:] in ['.s', '.S']:
						action = 'Assembl'
					else:  # elif self.fname[-2:] in ['.c','.C']:
						action = 'Compil'
					fname = '.'.join(self.fname.split('.')[:-1])
					if self.p.version == REGISTERS.VERSION_ATMEGA32 or self.p.connected == False:  # by default, compile for ATMEGA32.
						# cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" -Map "%s" "%s"' %('atmega32',fname,fname+'.map',self.fname)
						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -Wl,-Map="%s" -o "%s" "%s" %s' % (
							'atmega32', fname + '.map', fname, self.fname, "-lkp" if LKP else "")  # includes MAP
						self.logThis.emit('''<span style="color:#383;">%sing for Atmega32</span>''' % (action))
					elif self.p.version == REGISTERS.VERSION_ATMEGA328P:
						cmd = 'avr-gcc -Wall -O2 -mmcu=%s -o "%s" "%s"  %s' % (
						'atmega328p', fname, self.fname, "-lkp" if LKP else "")
						self.logThis.emit(
							'''<span style="color:#383;">%sing for Atmega328p (Nano)</span>''' % (action))
					else:
						self.logThis.emit('''<span style="color:#222;">%ser UNAVAILABLE</span>''' % (action))
						return
					print(cmd)
					res = subprocess.getstatusoutput(cmd)
					if res[0] != 0:
						self.logThis.emit('''<span style="color:#526;">%se Error: %s</span>''' % (
							action, res[1].replace('\n', '<br>')))
						self.finished.emit()
						return

					else:
						self.logThis.emit('''<span style="color:gray;">%s</span><br>''' % res[1])

					cmd = 'avr-objcopy -j .text -j .data -O ihex "%s" "%s.hex"' % (fname, fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit(res[1])  # '''<span style="color:gray;">%s</span><br>'''%res[1])

					cmd = 'avr-objdump -S "%s" > "%s.lst"' % (fname, fname)
					res = subprocess.getstatusoutput(cmd)
					self.logThis.emit(res[1])  # '''<span style="color:gray;">%s</span><br>'''%res[1])
					self.logThis.emit(
						'Finished %sing: Generated Hex File' % action)  # '''<span style="color:darkgreen;">Finished %sing: Generated Hex File</span>'''%(action))

					self.resultSignal.emit('list', fname + '.lst')
					self.resultSignal.emit('hex', fname + '.hex')
					self.resultSignal.emit('map', fname + '.map')
				except Exception as err:
					self.logThis.emit('''<span style="color:#d730ee;">Failed to %se:%s</span>''' % str(action, err))

			if self.p.connected:
				if 'upload' in self.mode:
					try:
						if self.fname[-2:] in ['.c', '.C', '.s', '.S']:
							self.fname = self.fname[:-2] + '.hex'  # Replace .c with .hex
						self.logThis.emit('''<span style="font-size:12pt">Upload Code... Trigger Reset...</span>''')
						dude = uploader.Uploader(self.p.fd, hexfile=self.fname, logger=self.logThis)
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.05)
						dude.program()
						dude.verify()
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.25)
						self.p.get_version()
						self.logThis.emit('''<span style="color:darkgreen;">Finished upload</span>''')
					except Exception as err:
						print('upload error', err)
						self.p.fd.setRTS(0);
						self.p.fd.setDTR(0);
						time.sleep(0.01);
						self.p.fd.setRTS(1);
						self.p.fd.setDTR(1);
						time.sleep(0.25)
						self.p.get_version()
						self.logThis.emit('''<span style="color:#d730ee;">Failed to upload</span>''')
			# self.jumpToApplication(False) #Force a reset
			self.finished.emit()

	def uploadHex(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self, " Open a hex file to upload to your KuttyPy", "",
		                                                 "Hex Files (*.hex)")
		if len(filename[0]):
			# self.userCode.setStyleSheet("border: 3px dashed #53ffff;")
			# self.tabs.setTabEnabled(0,False)
			self.uploadingHex = True
			self.log.clear()
			self.log.setText('''<span style="color:#225;">-- Uploading Code --</span><br>''')
			self.UploadObject.config('upload', self.p, filename[0])
			self.uploadThread.start()

	def openFile(self):
		filename = QtWidgets.QFileDialog.getOpenFileName(self, " Open a C/Asm file to edit", self.defaultDirectory,
		                                                 "C/ASM Files (*.c *.C *.s *.S);; Asm Files (*.s *.S);; C Files (*.c *.C)")
		if len(filename[0]):
			self.openFile_(filename[0])

	def openFile_(self, fname):
		for sourceTab in self.sourceTabs:
			print(self.sourceTabs[sourceTab][1], fname)
			if fname == self.sourceTabs[sourceTab][1]:  # File is already open
				self.activeSourceTab = sourceTab
				self.activeEditor = self.sourceTabs[sourceTab][0]
				self.CFile = self.sourceTabs[sourceTab][1]
				self.filenameLabel.setText(self.CFile)
				self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(sourceTab))
				return

		if self.CFile is not None:  # A file is altready open
			self.addSourceTab()
		# self.defaultDirectory = ''
		self.filenameLabel.setText(fname)
		self.CFile = fname
		self.defaultDirectory = os.path.split(self.CFile)[0]
		self.sourceTabs[self.activeSourceTab][1] = self.CFile
		self.log.clear()
		infile = open(fname, 'r')
		self.activeEditor.setPlainText(
			infile.read())  # self.activeEditor = self.sourceTabs[self.activeSourceTab][0]
		infile.close()
		self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
		self.log.setText('''<span style="color:#225;">-- Opened File: %s --</span><br>''' % fname)
		filetype = 'c'
		if self.CFile.endswith('.S') or self.CFile.endswith('.s'):
			filetype = 'asm'
		ico = QtGui.QIcon()
		ico.addPixmap(QtGui.QPixmap(f":/control/{filetype}.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.codingTabs.setTabIcon(self.codingTabs.indexOf(self.activeSourceTab), ico)
		self.updateWatcher()

	def updateWatcher(self):
		paths = []
		for sourceTab in self.sourceTabs:
			paths.append(self.sourceTabs[sourceTab][1])
		self.fs_watcher = QtCore.QFileSystemWatcher(paths)
		self.fs_watcher.fileChanged.connect(self.file_changed)
		print('updated watcher', paths)

	def file_changed(self, path):
		print('File Changed: %s' % path)
		self.reloadLabel.setText(path)
		self.reloadFrame.setVisible(True)

	def reloadFile(self):
		self.reloadFrame.setVisible(False)
		fname = self.reloadLabel.text()
		for sourceTab in self.sourceTabs:
			print(self.sourceTabs[sourceTab][1], fname)
			if fname == self.sourceTabs[sourceTab][1]:  # File is already open
				self.activeSourceTab = sourceTab
				self.activeEditor = self.sourceTabs[sourceTab][0]
				self.CFile = self.sourceTabs[sourceTab][1]
				self.filenameLabel.setText(self.CFile)
				self.codingTabs.setCurrentIndex(self.codingTabs.indexOf(sourceTab))
				self.defaultDirectory = os.path.split(self.CFile)[0]

				self.log.clear()
				infile = open(fname, 'r')
				self.activeEditor.setPlainText(
					infile.read())  # self.activeEditor = self.sourceTabs[self.activeSourceTab][0]
				infile.close()
				self.log.setText('''<span style="color:#225;">-- Reloaded File: %s --</span><br>''' % fname)
				filetype = 'c'
				if self.CFile.endswith('.S') or self.CFile.endswith('.s'):
					filetype = 'asm'
				ico = QtGui.QIcon()
				ico.addPixmap(QtGui.QPixmap(f":/control/{filetype}.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
				self.codingTabs.setTabIcon(self.codingTabs.indexOf(self.activeSourceTab), ico)
				self.updateWatcher()
				return
		print(' Modified file is not open anymore', fname)

	def cancelReload(self):
		self.reloadFrame.setVisible(False)
		self.updateWatcher()

	def fontPlus(self):
		size = self.editorFont.pointSize()
		if size > 40: return
		self.editorFont.setPointSize(size + 1)
		self.updateFont()

	def fontMinus(self):
		size = self.editorFont.pointSize()
		if size < 5: return
		self.editorFont.setPointSize(size - 1)
		self.updateFont()

	def setFont(self, font):
		self.editorFont.setFamily(font)
		self.updateFont()

	def updateFont(self):
		for editors in self.sourceTabs:
			self.sourceTabs[editors][0].setFont(self.editorFont)
		if hasattr(self, 'hexEditor') and self.hexEditor is not None:
			self.hexEditor.setFont(self.editorFont)
		if hasattr(self, 'listEditor') and self.listEditor is not None:
			self.listEditor.setFont(self.editorFont)
		if hasattr(self, 'mapEditor') and self.mapEditor is not None:
			self.mapEditor.setFont(self.editorFont)

	def compileAndUpload(self):
		self.setLogType('avr')
		self.saveFile()
		if self.CFile:
			if self.userHexRunning:
				self.launchFirmwareButton.setChecked(False)
				self.jumpToApplication(False)
			self.uploadingHex = True
			self.log.setText('''<span style="color:#225;">-- Compiling and Uploading Code --</span><br>''')
			self.UploadObject.config('compileupload', self.p, self.CFile)
			self.uploadThread.start()

	def compile(self):
		self.setLogType('avr')
		self.saveFile()
		if self.CFile:
			self.log.setText('''<span style="color:green;">-- Compiling Code --</span><br>''')
			self.log.setText('''<span style="color:green;">-- %s --</span><br>''' % self.CFile)
			self.UploadObject.config('compile', self.p, self.CFile)
			self.uploadThread.start()

	def upload(self):
		self.setLogType('avr')
		if self.CFile:
			if self.userHexRunning:
				self.launchFirmwareButton.setChecked(False)
				self.jumpToApplication(False)
			self.uploadingHex = True
			self.log.setText('''<span style="color:green;">-- Uploading Code --</span><br>''')
			self.UploadObject.config('upload', self.p, self.CFile)
			self.uploadThread.start()

	def saveFile(self):
		if not self.CFile:
			self.CFile = self.saveAs()
		if self.CFile is not None and len(self.CFile) > 1:
			self.sourceTabs[self.activeSourceTab][1] = self.CFile
			self.activeEditor.markAsSaved(True)
			self.fs_watcher.removePath(self.CFile)
			fn = open(self.CFile, 'w')
			fn.write(self.activeEditor.toPlainText())
			fn.close()
			self.fs_watcher.addPath(self.CFile)
			self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
			self.log.setText('''<span style="color:green;">-- Saved to: %s --</span><br>''' % self.CFile)
		else:
			self.log.setText('''<span style="color:#d730ee;">-- No File Selected --</span><br>''')

	def saveAs(self):
		name = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File')
		print('created new file:', name)
		if len(name) > 0 and len(name[0]) > 1:
			self.CFile = name[0]
			self.filenameLabel.setText(self.CFile)
			self.log.setText('''<span style="color:green;">-- Created new file: %s --</span><br>''' % self.CFile)
			self.codingTabs.setTabText(self.codingTabs.indexOf(self.activeSourceTab), os.path.split(self.CFile)[1])
			self.saveFile()
			return self.CFile

	##############################
	def setTheme(self, theme):
		self.setStyleSheet("")
		self.setStyleSheet(open(os.path.join(path["themes"], theme + ".qss"), "r").read())

	def initializeCommunications(self, port=False):
		if self.p:
			try:
				self.p.fd.close()
			except:
				pass
		if port:
			self.p = KuttyPyLib.connect(port=port)
		else:
			self.p = KuttyPyLib.connect(autoscan=True)

		if self.p.connected:
			self.VERSION = self.p.version
			self.launchFirmwareButton.setChecked(False)
			self.setWindowTitle('KuttyPy Integrated Development Environment [{0:s}]'.format(self.p.portname))

		else:
			self.setWindowTitle('KuttyPy Integrated Development Environment [ Hardware not detected ]')

	def setLanguage(self, lang='fr_FR'):
		translate(lang)
		self.retranslateUi(self)

	def showSerialGauge(self):
		self.serialGauge.show()

	def codeFinished(self):
		print('finished')
		self.uploadingHex = False

	def jumpToApplication(self, state):
		print('run firmware', state)
		if self.p:
			if state:
				self.serialFrame.show()
				self.userHexRunning = True
				self.p.fd.write(b'j')  # Skip to application (Bootloader resets)
				self.launchFirmwareButton.setText('Stop')

				self.setLogType('monitor')
				self.log.setText('''<span style="color:#004;">-- Serial Port Monitor --</span><br>''')
			# self.serialGauge.show()

			else:
				self.serialFrame.hide()
				self.p.fd.setRTS(0);
				self.p.fd.setDTR(0);
				time.sleep(0.01);
				self.p.fd.setRTS(1);
				self.p.fd.setDTR(1);
				time.sleep(0.2)
				while self.p.fd.in_waiting:
					self.p.fd.read()
				self.p.get_version()
				self.userHexRunning = False
				self.launchFirmwareButton.setText('Run')
				# self.tabs.setEnabled(True)
				self.serialGauge.hide()
		else:
			if self.isChecked():
				self.setChecked(False)

	def sendASCII(self):
		s = self.serialData.text()
		self.p.fd.write(s.encode('utf-8'))

	def sendBinary(self):
		s = self.serialData.text()
		try:
			self.p.fd.write(chr(int(s) & 0xFF).encode('utf-8'))
		except Exception as e:
			print(e)

	def showServerStatus(self, msg):
		self.showStatus("Compiler: Error Launching Server (Restart app) ", True)
		QtWidgets.QMessageBox.warning(self, 'Server Error', msg)
		self.compile_thread_button.setText("Online Compiler")

	# self.statusBar.addPermanentWidget(self.compile_thread_button)

	def showStatus(self, msg, error=None):
		if error:
			self.statusBar.setStyleSheet("color:#633")
		else:
			self.statusBar.setStyleSheet("color:#333")
		self.statusBar.showMessage(msg)

	def locateDevices(self):
		try:
			L = KuttyPyLib.getFreePorts(self.p.portname)
		except Exception as e:
			print(e)
		total = len(L)
		menuChanged = False
		if L != self.shortlist:
			menuChanged = True
			if self.p.connected:
				if self.p.portname not in L:
					self.setWindowTitle('Error : Device Disconnected')
					QtWidgets.QMessageBox.warning(self, 'Connection Error',
					                              'Device Disconnected. Please check the connections')
					try:
						self.p.close()
					except:
						pass
					self.p.connected = False
					self.setWindowTitle('KuttyPy Integrated Development Environment [ Hardware not detected ]')

			elif True in L.values():
				reply = QtWidgets.QMessageBox.question(self, 'Connection', 'Device Available. Connect?',
				                                       QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
				if reply == QtWidgets.QMessageBox.Yes:
					self.initializeCommunications()

			# update the shortlist
			self.shortlist = L

	def checkConnectionStatus(self, dialog=False):
		if self.p.connected:
			return True
		else:
			if dialog: QtWidgets.QMessageBox.warning(self, 'Connection Error',
			                                         'Device not connected. Please connect a KuttyPy to the USB port')
			return False

	######## WINDOW EXPORT SVG
	def exportSvg(self):
		from PyQt5 import QtSvg
		path, _filter = QtWidgets.QFileDialog.getSaveFileName(self, 'Save File', '~/')
		if path:
			generator = QtSvg.QSvgGenerator()
			generator.setFileName(path)
			target_rect = QtCore.QRectF(0, 0, 800, 600)
			generator.setSize(target_rect.size().toSize())  # self.size())
			generator.setViewBox(self.rect())
			generator.setTitle("Your title")
			generator.setDescription("some description")
			p = QtGui.QPainter()
			p.begin(generator)
			self.render(p)
			p.end()


def translators(langDir, lang=None):
	"""
    create a list of translators
    @param langDir a path containing .qm translation
    @param lang the preferred locale, like en_IN.UTF-8, fr_FR.UTF-8, etc.
    @result a list of QtCore.QTranslator instances
    """
	if lang == None:
		lang = QtCore.QLocale.system().name()
	result = []
	qtTranslator = QtCore.QTranslator()
	qtTranslator.load("qt_" + lang,
	                  QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath))
	result.append(qtTranslator)

	# path to the translation files (.qm files)
	sparkTranslator = QtCore.QTranslator()
	sparkTranslator.load(lang, langDir);
	result.append(sparkTranslator)
	return result


def firstExistingPath(l):
	"""
    Returns the first existing path taken from a list of
    possible paths.
    @param l a list of paths
    @return the first path which exists in the filesystem, or None
    """
	for p in l:
		if os.path.exists(p):
			return p
	return None


def common_paths():
	"""
    Finds common paths
    @result a dictionary of common paths
    """
	path = {}
	curPath = os.path.dirname(os.path.realpath(__file__))
	path["current"] = curPath
	sharedPath = "/usr/share/kuttypy"
	path["translation"] = firstExistingPath(
		[os.path.join(p, "lang") for p in
		 (curPath, sharedPath,)])
	path["utilities"] = firstExistingPath(
		[os.path.join(p, 'utilities') for p in
		 (curPath, sharedPath,)])
	path["templates"] = firstExistingPath(
		[os.path.join(p, 'utilities', 'templates') for p in
		 (curPath, sharedPath,)])

	path["themes"] = firstExistingPath(
		[os.path.join(p, 'utilities', 'themes') for p in
		 (curPath, sharedPath,)])

	path["examples"] = firstExistingPath(
		[os.path.join(p, 'examples') for p in
		 (curPath, sharedPath,)])

	path["kpy"] = firstExistingPath(
		[os.path.join(p, 'kpy') for p in
		 (curPath, sharedPath,)])

	path["editor"] = firstExistingPath(
		[os.path.join(p, 'editor') for p in
		 (curPath, sharedPath,)])

	lang = str(QtCore.QLocale.system().name())
	shortLang = lang[:2]
	return path


def run():
	global path, app, myapp
	path = common_paths()
	print('QT Version', QtWidgets.__file__)
	app = QtWidgets.QApplication(sys.argv)
	myapp = AppWindow(app=app, path=path)
	myapp.show()
	r = app.exec_()
	if myapp.compile_thread is not None:
		myapp.compile_thread.terminate()
		print('waiting to quit compile_thread')
		myapp.compile_thread.wait()

	app.deleteLater()
	sys.exit(r)


if __name__ == "__main__":
	run()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'regedit.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(364, 59)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Frame.sizePolicy().hasHeightForWidth())
        Frame.setSizePolicy(sizePolicy)
        Frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.gridLayout = QtWidgets.QGridLayout(Frame)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(1)
        self.gridLayout.setObjectName("gridLayout")
        self.b3 = QtWidgets.QCheckBox(Frame)
        self.b3.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b3.setText("")
        self.b3.setObjectName("b3")
        self.gridLayout.addWidget(self.b3, 0, 5, 1, 1)
        self.b2 = QtWidgets.QCheckBox(Frame)
        self.b2.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b2.setText("")
        self.b2.setObjectName("b2")
        self.gridLayout.addWidget(self.b2, 0, 6, 1, 1)
        self.regName = QtWidgets.QComboBox(Frame)
        self.regName.setEditable(True)
        self.regName.setMaxVisibleItems(15)
        self.regName.setObjectName("regName")
        self.gridLayout.addWidget(self.regName, 0, 0, 1, 1)
        self.b1 = QtWidgets.QCheckBox(Frame)
        self.b1.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b1.setText("")
        self.b1.setObjectName("b1")
        self.gridLayout.addWidget(self.b1, 0, 7, 1, 1)
        self.b0 = QtWidgets.QCheckBox(Frame)
        self.b0.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b0.setText("")
        self.b0.setObjectName("b0")
        self.gridLayout.addWidget(self.b0, 0, 8, 1, 1)
        self.b5 = QtWidgets.QCheckBox(Frame)
        self.b5.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b5.setText("")
        self.b5.setObjectName("b5")
        self.gridLayout.addWidget(self.b5, 0, 3, 1, 1)
        self.valueLabel = QtWidgets.QLabel(Frame)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.valueLabel.setFont(font)
        self.valueLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.valueLabel.setObjectName("valueLabel")
        self.gridLayout.addWidget(self.valueLabel, 1, 0, 1, 1)
        self.b7 = QtWidgets.QCheckBox(Frame)
        self.b7.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b7.setStyleSheet("")
        self.b7.setText("")
        self.b7.setObjectName("b7")
        self.gridLayout.addWidget(self.b7, 0, 1, 1, 1)
        self.b4 = QtWidgets.QCheckBox(Frame)
        self.b4.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b4.setText("")
        self.b4.setObjectName("b4")
        self.gridLayout.addWidget(self.b4, 0, 4, 1, 1)
        self.b6 = QtWidgets.QCheckBox(Frame)
        self.b6.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.b6.setText("")
        self.b6.setObjectName("b6")
        self.gridLayout.addWidget(self.b6, 0, 2, 1, 1)
        self.l7 = QtWidgets.QLabel(Frame)
        self.l7.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l7.setObjectName("l7")
        self.gridLayout.addWidget(self.l7, 1, 1, 1, 1)
        self.l6 = QtWidgets.QLabel(Frame)
        self.l6.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l6.setObjectName("l6")
        self.gridLayout.addWidget(self.l6, 1, 2, 1, 1)
        self.l5 = QtWidgets.QLabel(Frame)
        self.l5.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l5.setObjectName("l5")
        self.gridLayout.addWidget(self.l5, 1, 3, 1, 1)
        self.l4 = QtWidgets.QLabel(Frame)
        self.l4.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l4.setObjectName("l4")
        self.gridLayout.addWidget(self.l4, 1, 4, 1, 1)
        self.l3 = QtWidgets.QLabel(Frame)
        self.l3.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l3.setObjectName("l3")
        self.gridLayout.addWidget(self.l3, 1, 5, 1, 1)
        self.l2 = QtWidgets.QLabel(Frame)
        self.l2.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l2.setObjectName("l2")
        self.gridLayout.addWidget(self.l2, 1, 6, 1, 1)
        self.l1 = QtWidgets.QLabel(Frame)
        self.l1.setCursor(QtGui.QCursor(QtCore.Qt.SizeVerCursor))
        self.l1.setObjectName("l1")
        self.gridLayout.addWidget(self.l1, 1, 7, 1, 1)
        self.l0 = QtWidgets.QLabel(Frame)
        self.l0.setObjectName("l0")
        self.gridLayout.addWidget(self.l0, 1, 8, 1, 1)
        self.typeLabel = QtWidgets.QLabel(Frame)
        self.typeLabel.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.typeLabel.setProperty("clickable", True)
        self.typeLabel.setObjectName("typeLabel")
        self.gridLayout.addWidget(self.typeLabel, 0, 9, 1, 1)
        self.pushButton = QtWidgets.QPushButton(Frame)
        self.pushButton.setCursor(QtGui.QCursor(QtCore.Qt.PointingHandCursor))
        self.pushButton.setStyleSheet("")
        self.pushButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 1, 9, 1, 1)

        self.retranslateUi(Frame)
        self.pushButton.clicked.connect(Frame.execute) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        _translate = QtCore.QCoreApplication.translate
        Frame.setWindowTitle(_translate("Frame", "Frame"))
        self.valueLabel.setText(_translate("Frame", "0"))
        self.l7.setText(_translate("Frame", "7"))
        self.l6.setText(_translate("Frame", "6"))
        self.l5.setText(_translate("Frame", "5"))
        self.l4.setText(_translate("Frame", "4"))
        self.l3.setText(_translate("Frame", "3"))
        self.l2.setText(_translate("Frame", "2"))
        self.l1.setText(_translate("Frame", "1"))
        self.l0.setText(_translate("Frame", "0"))
        self.typeLabel.setText(_translate("Frame", "↑READ"))
from . import res_rc



#################  Arduino Uno R3 ##########

def genlist(**kwargs):
	return kwargs

VERSIONS = {}

########### 328 PU . Arduino Uno R3

REGISTERS = genlist(
TWAMR	= 0xBD,
TWCR	= 0xBC,
TWDR	= 0xBB,
TWAR	= 0xBA,
TWSR	= 0xB9,
TWBR	= 0xB8,
ASSR	= 0xB6,
OCR2B	= 0xB4,
OCR2A	= 0xB3,
TCNT2	= 0xB2,
TCCR2B	= 0xB1,
TCCR2A	= 0xB0,
OCR1BH	= 0x8B,
OCR1BL	= 0x8A,
OCR1AH	= 0x89,
OCR1AL	= 0x88,
ICR1H	= 0x87,
ICR1L	= 0x86,
TCNT1H	= 0x85,
TCNT1L	= 0x84,
TCCR1C	= 0x82,
TCCR1B	= 0x81,
TCCR1A	= 0x80,
DIDR1	= 0x7F,
DIDR0	= 0x7E,
ADMUX	= 0x7C,
ADCSRB	= 0x7B,
ADCSRA	= 0x7A,
ADCH	= 0x79,
ADCL	= 0x78,
TIMSK2	= 0x70,
TIMSK1	= 0x6F,
TIMSK0	= 0x6E,
OCR0B	= 0x48,
OCR0A	= 0x47,
TCNT0	= 0x46,
TCCR0B	= 0x45,
TCCR0A	= 0x44,
GTCCR	= 0x43,
EEARH	= 0x42,
EEARL	= 0x41,
EEDR	= 0x40,
EECR	= 0x3F,
GPIOR0	= 0x3E,
EIMSK	= 0x3D,
EIFR	= 0x3C,
TIFR2	= 0x37,
TIFR1	= 0x36,
TIFR0	= 0x35,
PORTD	= 0x2B,
DDRD	= 0x2A,
PIND	= 0x29,
PORTC	= 0x28,
DDRC	= 0x27,
PINC	= 0x26,
PORTB	= 0x25,
DDRB	= 0x24,
PINB	= 0x23,

)


SPECIALS = genlist(
PB1 = 'OC1A',
PB2 = 'OC1B',
PB3 = 'OC2',
PD4 = 'T0',
PD5 = 'T1',
PC0 = 'ADC0',
PC1 = 'ADC1',
PC2 = 'ADC2',
PC3 = 'ADC3',
)
for a in ['GND','VCC','RESET','5V','3V3','SCL','SDA','AREF','ADC6','ADC7','VIN']:
	SPECIALS[a] = 'FIXED'
################################
VERSIONNUM = 100
ADC = ['PC0','PC1','PC2','PC3','PC4','PC5']
EXAMPLES = 'atmega328p'
RESTRICTED = ['UBRRL']









# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layoutuno.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(754, 635)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/kuttypy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet("")
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabs = QtWidgets.QTabWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabs.sizePolicy().hasHeightForWidth())
        self.tabs.setSizePolicy(sizePolicy)
        self.tabs.setObjectName("tabs")
        self.playground = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.playground.sizePolicy().hasHeightForWidth())
        self.playground.setSizePolicy(sizePolicy)
        self.playground.setObjectName("playground")
        self.gridLayout = QtWidgets.QGridLayout(self.playground)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.playground)
        self.label.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label.setScaledContents(True)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(64, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 2, 1, 1)
        self.tabs.addTab(self.playground, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(51, 24, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 0, 1, 1, 1)
        self.registerList = QtWidgets.QListWidget(self.tab)
        self.registerList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.registerList.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.registerList.setObjectName("registerList")
        self.gridLayout_4.addWidget(self.registerList, 1, 0, 1, 3)
        self.pushButton = QtWidgets.QPushButton(self.tab)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/control/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon1)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_4.addWidget(self.pushButton, 0, 0, 1, 1)
        self.autoRefreshUserRegisters = QtWidgets.QCheckBox(self.tab)
        self.autoRefreshUserRegisters.setObjectName("autoRefreshUserRegisters")
        self.gridLayout_4.addWidget(self.autoRefreshUserRegisters, 0, 2, 1, 1)
        self.tabs.addTab(self.tab, "")
        self.examples = QtWidgets.QWidget()
        self.examples.setObjectName("examples")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.examples)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setHorizontalSpacing(3)
        self.gridLayout_3.setVerticalSpacing(2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.toolButton = QtWidgets.QToolButton(self.examples)
        self.toolButton.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/control/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton.setIcon(icon2)
        self.toolButton.setObjectName("toolButton")
        self.gridLayout_3.addWidget(self.toolButton, 0, 2, 1, 1)
        self.exampleList = QtWidgets.QComboBox(self.examples)
        self.exampleList.setObjectName("exampleList")
        self.gridLayout_3.addWidget(self.exampleList, 0, 0, 1, 1)
        self.userCode = QtWidgets.QPlainTextEdit(self.examples)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.userCode.setFont(font)
        self.userCode.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.userCode.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.userCode.setTabStopWidth(30)
        self.userCode.setBackgroundVisible(False)
        self.userCode.setCenterOnScroll(False)
        self.userCode.setProperty("class", "")
        self.userCode.setObjectName("userCode")
        self.gridLayout_3.addWidget(self.userCode, 1, 0, 1, 3)
        self.toolButton_2 = QtWidgets.QToolButton(self.examples)
        self.toolButton_2.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/control/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton_2.setIcon(icon3)
        self.toolButton_2.setObjectName("toolButton_2")
        self.gridLayout_3.addWidget(self.toolButton_2, 0, 1, 1, 1)
        self.pushButton_2 = QtWidgets.QPushButton(self.examples)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout_3.addWidget(self.pushButton_2, 2, 0, 1, 3)
        self.tabs.addTab(self.examples, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.filenameLabel = QtWidgets.QLabel(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filenameLabel.sizePolicy().hasHeightForWidth())
        self.filenameLabel.setSizePolicy(sizePolicy)
        self.filenameLabel.setObjectName("filenameLabel")
        self.gridLayout_5.addWidget(self.filenameLabel, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem3, 2, 0, 1, 1)
        self.commandLinkButton_2 = QtWidgets.QCommandLinkButton(self.tab_2)
        self.commandLinkButton_2.setObjectName("commandLinkButton_2")
        self.gridLayout_5.addWidget(self.commandLinkButton_2, 0, 0, 1, 1)
        self.commandLinkButton_3 = QtWidgets.QCommandLinkButton(self.tab_2)
        self.commandLinkButton_3.setObjectName("commandLinkButton_3")
        self.gridLayout_5.addWidget(self.commandLinkButton_3, 1, 0, 1, 1)
        self.tabs.addTab(self.tab_2, "")
        self.gridLayout_2.addWidget(self.tabs, 0, 0, 1, 6)
        self.enableLog = QtWidgets.QCheckBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.enableLog.sizePolicy().hasHeightForWidth())
        self.enableLog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setItalic(True)
        self.enableLog.setFont(font)
        self.enableLog.setChecked(True)
        self.enableLog.setObjectName("enableLog")
        self.gridLayout_2.addWidget(self.enableLog, 6, 0, 1, 1)
        self.log = QtWidgets.QTextBrowser(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log.sizePolicy().hasHeightForWidth())
        self.log.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.log.setFont(font)
        self.log.setLineWrapMode(QtWidgets.QTextEdit.WidgetWidth)
        self.log.setObjectName("log")
        self.gridLayout_2.addWidget(self.log, 3, 0, 2, 6)
        self.logLabel = QtWidgets.QLabel(self.centralwidget)
        self.logLabel.setMinimumSize(QtCore.QSize(0, 30))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setItalic(True)
        self.logLabel.setFont(font)
        self.logLabel.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.logLabel.setObjectName("logLabel")
        self.gridLayout_2.addWidget(self.logLabel, 2, 0, 1, 6)
        self.clearLog = QtWidgets.QCheckBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clearLog.sizePolicy().hasHeightForWidth())
        self.clearLog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setItalic(True)
        self.clearLog.setFont(font)
        self.clearLog.setChecked(True)
        self.clearLog.setObjectName("clearLog")
        self.gridLayout_2.addWidget(self.clearLog, 6, 1, 1, 1)
        self.toolButton_4 = QtWidgets.QToolButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/control/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton_4.setIcon(icon4)
        self.toolButton_4.setObjectName("toolButton_4")
        self.gridLayout_2.addWidget(self.toolButton_4, 6, 5, 1, 1)
        self.frame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.sensorLayout = QtWidgets.QHBoxLayout(self.frame)
        self.sensorLayout.setContentsMargins(0, 0, 0, 0)
        self.sensorLayout.setSpacing(2)
        self.sensorLayout.setObjectName("sensorLayout")
        self.gridLayout_2.addWidget(self.frame, 6, 2, 1, 3)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.leftdock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.leftdock.sizePolicy().hasHeightForWidth())
        self.leftdock.setSizePolicy(sizePolicy)
        self.leftdock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.leftdock.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.leftdock.setWindowTitle("")
        self.leftdock.setObjectName("leftdock")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.leftlayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.leftlayout.setObjectName("leftlayout")
        self.leftdock.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.leftdock)
        self.rightdock = QtWidgets.QDockWidget(MainWindow)
        self.rightdock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.rightdock.setAllowedAreas(QtCore.Qt.RightDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.rightdock.setWindowTitle("")
        self.rightdock.setObjectName("rightdock")
        self.dockWidgetContents_5 = QtWidgets.QWidget()
        self.dockWidgetContents_5.setObjectName("dockWidgetContents_5")
        self.rightlayout = QtWidgets.QVBoxLayout(self.dockWidgetContents_5)
        self.rightlayout.setObjectName("rightlayout")
        self.rightdock.setWidget(self.dockWidgetContents_5)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.rightdock)

        self.retranslateUi(MainWindow)
        self.tabs.setCurrentIndex(0)
        self.toolButton.clicked.connect(MainWindow.runCode) # type: ignore
        self.tabs.currentChanged['int'].connect(MainWindow.tabChanged) # type: ignore
        self.toolButton_2.clicked.connect(MainWindow.abort) # type: ignore
        self.exampleList.currentIndexChanged['QString'].connect(MainWindow.loadExample) # type: ignore
        self.pushButton.clicked.connect(MainWindow.newRegister) # type: ignore
        self.autoRefreshUserRegisters.clicked['bool'].connect(MainWindow.userRegistersAutoRefresh) # type: ignore
        self.commandLinkButton_2.clicked.connect(MainWindow.openFile) # type: ignore
        self.commandLinkButton_3.clicked.connect(MainWindow.compileAndUpload) # type: ignore
        self.toolButton_4.clicked.connect(MainWindow.I2CScan) # type: ignore
        self.pushButton_2.clicked.connect(MainWindow.ipython) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "KuttyPy Interactive Console"))
        self.label.setText(_translate("MainWindow", "<html><head/><body><p><img src=\":/control/unopinout.png\"/></p></body></html>"))
        self.tabs.setTabText(self.tabs.indexOf(self.playground), _translate("MainWindow", "Playground"))
        self.pushButton.setText(_translate("MainWindow", "Add A New Register"))
        self.autoRefreshUserRegisters.setText(_translate("MainWindow", "Auto-Refresh"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), _translate("MainWindow", "Registers"))
        self.userCode.setPlainText(_translate("MainWindow", "import time\n"
"from kuttyPy import *\n"
"setReg(DDRD,255)\n"
"\n"
"for a in range(5):   #Run this loop 5 times. \n"
"    setReg(PORTD, 255)\n"
"    time.sleep(0.5)\n"
"    setReg(PORTD, 0)\n"
"    time.sleep(0.5)"))
        self.pushButton_2.setText(_translate("MainWindow", "Launch iPython Console"))
        self.tabs.setTabText(self.tabs.indexOf(self.examples), _translate("MainWindow", "Scripting"))
        self.filenameLabel.setText(_translate("MainWindow", ":"))
        self.commandLinkButton_2.setText(_translate("MainWindow", "Open File"))
        self.commandLinkButton_3.setText(_translate("MainWindow", "Compile+Upload"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), _translate("MainWindow", "C Code"))
        self.enableLog.setText(_translate("MainWindow", "Enabled"))
        self.logLabel.setText(_translate("MainWindow", "Monitor registers being read and set during each operation"))
        self.clearLog.setText(_translate("MainWindow", "Auto-Clear"))
        self.toolButton_4.setText(_translate("MainWindow", "..."))
from . import uno_rc



import os
from . import build_details
ENVIRON = build_details.QT_VERSION

os.environ['PYQTGRAPH_QT_LIB'] = ENVIRON
os.environ['CSMCA_QT_LIB'] = ENVIRON

#For remote operation
#os.environ['DATABASE_URL'] = 'postgres:///eyes_db'



def genlist(**kwargs):
	return kwargs

VERSIONS = {}

ATMEGA32_REGISTERS = genlist(
UBRRL	= 0X29,
UCSRB	= 0X2A,
UCSRA	= 0X2B,
UDR	= 0X2C,
SPCR	= 0X2D,
SPSR	= 0X2E,
SPDR	= 0X2F,
PIND	= 0x30,       # Port D input
DDRD	= 0x31,	# Port D direction
PORTD	= 0x32,		# Port D output
PINC	= 0x33,
DDRC	= 0x34,
PORTC	= 0x35,
PINB	= 0x36,
DDRB	= 0x37,
PORTB	= 0x38,
PINA	= 0x39,
DDRA	= 0x3A,
PORTA	= 0x3B,
EECR	= 0X3C,
EEDR	= 0X3D,
EEARL	= 0X3E,
EEARH	= 0X3F,

TWBR    = 0X20,
TWSR    = 0X21,
TWAR    = 0X22,
TWDR    = 0X23,

ADCL	= 0X24,       # ADC data
ADCH	= 0X25,
ADCSRA	= 0X26,	# ADC status/control
ADMUX	= 0X27,    # ADC channel, reference
ACSR	= 0X28,



OCR2	= 0X43,		# Timer/Counter 2  Output Compare  Reg
TCNT2	= 0X44	,	# Counter2 
TCCR2	= 0x45,		# Timer/Counter 2 control reg
ICR1L	= 0X46,
ICR1H	= 0X47,
OCR1BL	= 0X48,
OCR1BH	= 0X49,
OCR1AL	= 0X4A,
OCR1AH	= 0X4B,
TCNT1L	= 0X4C,
TCNT1H	= 0X4D,
TCCR1B	= 0X4E,
TCCR1A	= 0x4F,
SFIOR	= 0X50,

TCNT0	= 0x52	,	# Timer/ Counter 0
TCCR0	= 0x53,
MCUCSR	= 0X54,
MCUCR	= 0X55,
TWCR	= 0X56,
SPMCR	= 0X57,
TIFR	= 0X58,
TIMSK	= 0X59,
GIFR	= 0X5A,
GICR	= 0X5B,
OCR0	= 0x5C,
SPL		= 0X5D,
SPH		= 0x5E,
SREG	= 0X5F,
)


ATMEGA32_SPECIALS = genlist(
PD5 = 'OC1A',
PD7 = 'OC2',
PB3 = 'OC0',
PB1 = 'T1',
PA0 = 'ADC0',
PA1 = 'ADC1',
PA2 = 'ADC2',
PA3 = 'ADC3',
PA4 = 'ADC4',
PA5 = 'ADC5',
PA6 = 'ADC6',
PA7 = 'ADC7',
)
################################
VERSION_ATMEGA32 = 99
VERSIONS[VERSION_ATMEGA32] = {
'REGISTERS':ATMEGA32_REGISTERS,
'SPECIALS':ATMEGA32_SPECIALS,
'examples directory':'atmega32',
'ADC':['PA0','PA1','PA2','PA3','PA4','PA5','PA6','PA7'],
'RESTRICTED_REGISTERS':['UBRRL']
}




########### 328 P

ATMEGA328P_REGISTERS = genlist(
TWAMR	= 0xBD,
TWCR	= 0xBC,
TWDR	= 0xBB,
TWAR	= 0xBA,
TWSR	= 0xB9,
TWBR	= 0xB8,
ASSR	= 0xB6,
OCR2B	= 0xB4,
OCR2A	= 0xB3,
TCNT2	= 0xB2,
TCCR2B	= 0xB1,
TCCR2A	= 0xB0,
OCR1BH	= 0x8B,
OCR1BL	= 0x8A,
OCR1AH	= 0x89,
OCR1AL	= 0x88,
ICR1H	= 0x87,
ICR1L	= 0x86,
TCNT1H	= 0x85,
TCNT1L	= 0x84,
TCCR1C	= 0x82,
TCCR1B	= 0x81,
TCCR1A	= 0x80,
DIDR1	= 0x7F,
DIDR0	= 0x7E,
ADMUX	= 0x7C,
ADCSRB	= 0x7B,
ADCSRA	= 0x7A,
ADCH	= 0x79,
ADCL	= 0x78,
TIMSK2	= 0x70,
TIMSK1	= 0x6F,
TIMSK0	= 0x6E,
OCR0B	= 0x48,
OCR0A	= 0x47,
TCNT0	= 0x46,
TCCR0B	= 0x45,
TCCR0A	= 0x44,
GTCCR	= 0x43,
EEARH	= 0x42,
EEARL	= 0x41,
EEDR	= 0x40,
EECR	= 0x3F,
GPIOR0	= 0x3E,
EIMSK	= 0x3D,
EIFR	= 0x3C,
TIFR2	= 0x37,
TIFR1	= 0x36,
TIFR0	= 0x35,
PORTD	= 0x2B,
DDRD	= 0x2A,
PIND	= 0x29,
PORTC	= 0x28,
DDRC	= 0x27,
PINC	= 0x26,
PORTB	= 0x25,
DDRB	= 0x24,
PINB	= 0x23,

)


ATMEGA328P_SPECIALS = genlist(
PB1 = 'OC1A',
PB2 = 'OC1B',
PB3 = 'OC2',
PD4 = 'T0',
PD5 = 'T1',
PC0 = 'ADC0',
PC1 = 'ADC1',
PC2 = 'ADC2',
PC3 = 'ADC3',
)
for a in ['GND','VCC','RESET','5V','3V3','SCL','SDA','AREF','ADC6','ADC7','VIN']:
	ATMEGA328P_SPECIALS[a] = 'FIXED'
################################
VERSION_ATMEGA328P = 100
VERSIONS[VERSION_ATMEGA328P] = {
'REGISTERS':ATMEGA328P_REGISTERS,
'SPECIALS':ATMEGA328P_SPECIALS,
'ADC':['PC0','PC1','PC2','PC3','PC4','PC5'],
'examples directory':'atmega328p',
'RESTRICTED_REGISTERS':['UBRRL']
}








# syntax.py

import sys

from PyQt5 import QtCore, QtGui, QtWidgets


def format(color, style=''):
	"""Return a QTextCharFormat with the given attributes.
    """
	_color = QtGui.QColor()
	_color.setNamedColor(color)

	_format = QtGui.QTextCharFormat()
	_format.setForeground(_color)
	if 'bold' in style:
		_format.setFontWeight(QtGui.QFont.Bold)
	if 'italic' in style:
		_format.setFontItalic(True)

	return _format


# Syntax styles that can be shared by all languages
STYLES = {
	'keyword': format('blue'),
	'operator': format('red'),
	'brace': format('darkGray'),
	'defclass': format('black', 'bold'),
	'string': format('magenta'),
	'string2': format('darkMagenta'),
	'comment': format('darkGreen', 'italic'),
	'self': format('black', 'italic'),
	'numbers': format('brown'),
}


class PythonHighlighter(QtGui.QSyntaxHighlighter):
	"""Syntax highlighter for the Python language.
    """
	# Python keywords
	keywords = [
		'and', 'assert', 'break', 'class', 'continue', 'def',
		'del', 'elif', 'else', 'except', 'exec', 'finally',
		'for', 'from', 'global', 'if', 'import', 'in',
		'is', 'lambda', 'not', 'or', 'pass', 'print',
		'raise', 'return', 'try', 'while', 'yield',
		'None', 'True', 'False', 'define', 'include'
	]

	# Python operators
	operators = [
		'=',
		# Comparison
		'==', '!=', '<', '<=', '>', '>=',
		# Arithmetic
		'\+', '-', '\*', '/', '//', '\%', '\*\*',
		# In-place
		'\+=', '-=', '\*=', '/=', '\%=',
		# Bitwise
		'\^', '\|', '\&', '\~', '>>', '<<',
	]

	# Python braces
	braces = [
		'\{', '\}', '\(', '\)', '\[', '\]',
	]

	def __init__(self, parent: QtGui.QTextDocument) -> None:
		super().__init__(parent)

		# Multi-line strings (expression, flag, style)
		self.tri_single = (QtCore.QRegExp("'''"), 1, STYLES['string2'])
		self.tri_double = (QtCore.QRegExp('"""'), 2, STYLES['string2'])

		rules = []

		# Keyword, operator, and brace rules
		rules += [(r'\b%s\b' % w, 0, STYLES['keyword'])
		          for w in PythonHighlighter.keywords]
		rules += [(r'%s' % o, 0, STYLES['operator'])
		          for o in PythonHighlighter.operators]
		rules += [(r'%s' % b, 0, STYLES['brace'])
		          for b in PythonHighlighter.braces]

		# All other rules
		rules += [
			# 'self'
			(r'\bself\b', 0, STYLES['self']),

			# 'def' followed by an identifier
			(r'\bvoid\b\s*(\w+)', 1, STYLES['defclass']),
			# 'class' followed by an identifier
			(r'\bint\b\s*(\w+)', 1, STYLES['defclass']),

			# Numeric literals
			(r'\b[+-]?[0-9]+[lL]?\b', 0, STYLES['numbers']),
			(r'\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b', 0, STYLES['numbers']),
			(r'\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b', 0, STYLES['numbers']),

			# Double-quoted string, possibly containing escape sequences
			(r'"[^"\\]*(\\.[^"\\]*)*"', 0, STYLES['string']),
			# Single-quoted string, possibly containing escape sequences
			(r"'[^'\\]*(\\.[^'\\]*)*'", 0, STYLES['string']),

			# From '#' until a newline
			(r'//[^\n]*', 0, STYLES['comment']),
		]

		# Build a QRegExp for each pattern
		self.rules = [(QtCore.QRegExp(pat), index, fmt)
		              for (pat, index, fmt) in rules]

	def highlightBlock(self, text):
		"""Apply syntax highlighting to the given block of text.
        """
		self.tripleQuoutesWithinStrings = []
		# Do other syntax formatting
		for expression, nth, format in self.rules:
			index = expression.indexIn(text, 0)
			if index >= 0:
				# if there is a string we check
				# if there are some triple quotes within the string
				# they will be ignored if they are matched again
				if expression.pattern() in [r'"[^"\\]*(\\.[^"\\]*)*"', r"'[^'\\]*(\\.[^'\\]*)*'"]:
					innerIndex = self.tri_single[0].indexIn(text, index + 1)
					if innerIndex == -1:
						innerIndex = self.tri_double[0].indexIn(text, index + 1)

					if innerIndex != -1:
						tripleQuoteIndexes = range(innerIndex, innerIndex + 3)
						self.tripleQuoutesWithinStrings.extend(tripleQuoteIndexes)

			while index >= 0:
				# skipping triple quotes within strings
				if index in self.tripleQuoutesWithinStrings:
					index += 1
					expression.indexIn(text, index)
					continue

				# We actually want the index of the nth match
				index = expression.pos(nth)
				length = len(expression.cap(nth))
				self.setFormat(index, length, format)
				index = expression.indexIn(text, index + length)

		self.setCurrentBlockState(0)

		# Do multi-line strings
		in_multiline = self.match_multiline(text, *self.tri_single)
		if not in_multiline:
			in_multiline = self.match_multiline(text, *self.tri_double)

	def match_multiline(self, text, delimiter, in_state, style):
		"""Do highlighting of multi-line strings. ``delimiter`` should be a
        ``QRegExp`` for triple-single-quotes or triple-double-quotes, and
        ``in_state`` should be a unique integer to represent the corresponding
        state changes when inside those strings. Returns True if we're still
        inside a multi-line string when this function is finished.
        """
		# If inside triple-single quotes, start at 0
		if self.previousBlockState() == in_state:
			start = 0
			add = 0
		# Otherwise, look for the delimiter on this line
		else:
			start = delimiter.indexIn(text)
			# skipping triple quotes within strings
			if start in self.tripleQuoutesWithinStrings:
				return False
			# Move past this match
			add = delimiter.matchedLength()

		# As long as there's a delimiter match on this line...
		while start >= 0:
			# Look for the ending delimiter
			end = delimiter.indexIn(text, start + add)
			# Ending delimiter on this line?
			if end >= add:
				length = end - start + add + delimiter.matchedLength()
				self.setCurrentBlockState(0)
			# No; multi-line string
			else:
				self.setCurrentBlockState(in_state)
				length = len(text) - start + add
			# Apply formatting
			self.setFormat(start, length, style)
			# Look for the next match
			start = delimiter.indexIn(text, start + length)

		# Return True if still inside a multi-line string, False otherwise
		if self.currentBlockState() == in_state:
			return True
		else:
			return False


class CHighlighter(QtGui.QSyntaxHighlighter):
	"""Syntax highlighter for the C language.
    """

	# C keywords
	keywords = [
		'and', 'assert', 'break', 'class', 'continue', 'def',
		'del', 'elif', 'else', 'except', 'exec', 'finally',
		'for', 'from', 'global', 'if', 'import', 'in',
		'is', 'lambda', 'not', 'or', 'pass', 'print',
		'raise', 'return', 'try', 'while', 'yield',
		'None', 'True', 'False', 'define', 'include', 'uint16_t', 'uint8_t'
	]

	# Python operators
	operators = [
		'=',
		# Comparison
		'==', '!=', '<', '<=', '>', '>=',
		# Arithmetic
		'\+', '-', '\*', '/', '//', '\%', '\*\*',
		# In-place
		'\+=', '-=', '\*=', '/=', '\%=',
		# Bitwise
		'\^', '\|', '\&', '\~', '>>', '<<',
	]

	# Python braces
	braces = [
		'\{', '\}', '\(', '\)', '\[', '\]',
	]

	def __init__(self, parent: QtGui.QTextDocument) -> None:
		super().__init__(parent)

		# Multi-line strings (expression, flag, style)
		self.tri_single = (QtCore.QRegExp("'''"), 1, STYLES['string2'])
		self.tri_double = (QtCore.QRegExp('"""'), 2, STYLES['string2'])

		rules = []

		# Keyword, operator, and brace rules
		rules += [(r'\b%s\b' % w, 0, STYLES['keyword'])
		          for w in self.keywords]
		rules += [(r'%s' % o, 0, STYLES['operator'])
		          for o in self.operators]
		rules += [(r'%s' % b, 0, STYLES['brace'])
		          for b in self.braces]

		# All other rules
		rules += [
			# 'self'
			(r'\bself\b', 0, STYLES['self']),

			# 'def' followed by an identifier
			(r'\bvoid\b\s*(\w+)', 1, STYLES['defclass']),
			# 'class' followed by an identifier
			(r'\bint\b\s*(\w+)', 1, STYLES['defclass']),

			# Numeric literals
			(r'\b[+-]?[0-9]+[lL]?\b', 0, STYLES['numbers']),
			(r'\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b', 0, STYLES['numbers']),
			(r'\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b', 0, STYLES['numbers']),

			# Double-quoted string, possibly containing escape sequences
			(r'"[^"\\]*(\\.[^"\\]*)*"', 0, STYLES['string']),
			# Single-quoted string, possibly containing escape sequences
			(r"'[^'\\]*(\\.[^'\\]*)*'", 0, STYLES['string']),

			# From '#' until a newline
			(r'//[^\n]*', 0, STYLES['comment']),
		]

		# Build a QRegExp for each pattern
		self.rules = [(QtCore.QRegExp(pat), index, fmt)
		              for (pat, index, fmt) in rules]

	def highlightBlock(self, text):
		"""Apply syntax highlighting to the given block of text.
        """
		self.tripleQuoutesWithinStrings = []
		# Do other syntax formatting
		for expression, nth, format in self.rules:
			index = expression.indexIn(text, 0)
			if index >= 0:
				# if there is a string we check
				# if there are some triple quotes within the string
				# they will be ignored if they are matched again
				if expression.pattern() in [r'"[^"\\]*(\\.[^"\\]*)*"', r"'[^'\\]*(\\.[^'\\]*)*'"]:
					innerIndex = self.tri_single[0].indexIn(text, index + 1)
					if innerIndex == -1:
						innerIndex = self.tri_double[0].indexIn(text, index + 1)

					if innerIndex != -1:
						tripleQuoteIndexes = range(innerIndex, innerIndex + 3)
						self.tripleQuoutesWithinStrings.extend(tripleQuoteIndexes)

			while index >= 0:
				# skipping triple quotes within strings
				if index in self.tripleQuoutesWithinStrings:
					index += 1
					expression.indexIn(text, index)
					continue

				# We actually want the index of the nth match
				index = expression.pos(nth)
				length = len(expression.cap(nth))
				self.setFormat(index, length, format)
				index = expression.indexIn(text, index + length)

		self.setCurrentBlockState(0)

		# Do multi-line strings
		in_multiline = self.match_multiline(text, *self.tri_single)
		if not in_multiline:
			in_multiline = self.match_multiline(text, *self.tri_double)

	def match_multiline(self, text, delimiter, in_state, style):
		"""Do highlighting of multi-line strings. ``delimiter`` should be a
        ``QRegExp`` for triple-single-quotes or triple-double-quotes, and
        ``in_state`` should be a unique integer to represent the corresponding
        state changes when inside those strings. Returns True if we're still
        inside a multi-line string when this function is finished.
        """
		# If inside triple-single quotes, start at 0
		if self.previousBlockState() == in_state:
			start = 0
			add = 0
		# Otherwise, look for the delimiter on this line
		else:
			start = delimiter.indexIn(text)
			# skipping triple quotes within strings
			if start in self.tripleQuoutesWithinStrings:
				return False
			# Move past this match
			add = delimiter.matchedLength()

		# As long as there's a delimiter match on this line...
		while start >= 0:
			# Look for the ending delimiter
			end = delimiter.indexIn(text, start + add)
			# Ending delimiter on this line?
			if end >= add:
				length = end - start + add + delimiter.matchedLength()
				self.setCurrentBlockState(0)
			# No; multi-line string
			else:
				self.setCurrentBlockState(in_state)
				length = len(text) - start + add
			# Apply formatting
			self.setFormat(start, length, style)
			# Look for the next match
			start = delimiter.indexIn(text, start + length)

		# Return True if still inside a multi-line string, False otherwise
		if self.currentBlockState() == in_state:
			return True
		else:
			return False
